# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2009, 2012 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
# IBM_COPYRIGHT_END 

# 
# 09N6996 
# 

sm_log () {
    _smha_log "$*"	
}

sm_msg () {
	
	local progname	
	if [ -n "$EXTPROGNAME" ]; then
	    progname=$EXTPROGNAME
	else
	    progname=$PROGNAME
	fi
	# Put it in the system log
	logger -t "$progname" "$*"
	# Put it on the our log
    sm_log "$*"	
    # Put it to stdout
    echo -e "$progname: $*"	
}

fatal_error_rc () {
    # The first parameter must be the exit code
    local RC=$1
    shift
    
    local progname  
    if [ -n "$EXTPROGNAME" ]; then
        progname=$EXTPROGNAME
    else
        progname=$PROGNAME
    fi

    
    if [[ "$LOGCOMMAND" == "ON" ]]; then         
        log_command_failed
        # Put it to stderr
        echo -e "$progname: $*" 1>&2
    else
        # Put in the system log and stderr
        logger -s -t "$progname" "$*"
    fi
    
    sm_log "$*"  
    
    exit $RC
}

fatal_error () {
    fatal_error_rc 1 $*    
}


# return_error is meant to be used from scripts that can not be 
# called directly by the user (scripts like smhainit.sh that are
# meant to be called by Director Server).  If return_error is 
# going to be called by a script that the user calls, such
# as smha or smhastatus (or any script that those scripts call
# in to), then the message passed into return_error should be
# translated using eval_gettext or something similar
# 
return_error () {
    # The first parameter must be the exit code    
    local RC=$1
    shift
    
    # Put in the system log and stderr
    logger -s -t "$PROGNAME" "$*"
    sm_log "$*"  #logger above will send to standard error
    exit $RC
}


sm_sys_log () {	
	smha_sys_log "${1}"
}


debug () {
    smha_debug "$*"
}

debug_tsa () {
    smha_debug_tsa "$*"
}

record_tsa_action () {
    smha_record_tsa_action $@
}

debug_if_not_null () {
    if [ -n "$*" ]; then
    	debug $*   
    fi	
}

mk_log_dir () {
    if [ ! -d $LOG_DIR ]; then
        mkdir -p $LOG_DIR
    fi  
}

# Returns 1 if this option should not be saved
# Parameters
#   option to check
# Relies on
#   PARM_DONT_SAVE
never_save_opt () {
	local i
	# if there is a list of options not to save
	if [ -n "$PARM_DONT_SAVE" ]; then	
	   for i in ${PARM_DONT_SAVE}
	   do
		  if [[ "$i" == "$1" ]]; then
			 return 1
		  fi
	   done
    fi
	return 0
	
}


# Append the set of variables to the cfg file
# Parameters are:
#   variable names
# Return value:
#  
append_to_cfg_file () {
	local i
	
	for i in $*
	do
		# convert the variable name back to an option name
		local option_name=${i//_/-}
		
		# if this is a multiple parm, then delete the __1 part
		option_name=${option_name%%--*}
		
		# Don't save this option if we shouldn't
		never_save_opt $option_name
		if [ $? -ne 1 ]; then
    		# write the option and value
	       	echo "--${option_name}=${!i}" >> $SMHA_CFG_FILE
		fi
	done
	return 0
}

# Append the string to the cfg file
# Parameters are:
#   string to append
# Return value:
#  
append_string_to_cfg_file () {
    local i
    
    for i in $*
    do
        echo "${i}" >> $SMHA_CFG_FILE
    done
    return 0
}


# Remove the cfg file
# Parameters are:
#   none
# Return value:
#  
remove_cfg_file () {
    if [ -f $SMHA_CFG_FILE ]; then
        rm $SMHA_CFG_FILE
    fi
}

# Create a new cfg file
# Parameters are:
#   variable names
# Return value:
#  
create_cfg_file () {
    remove_cfg_file
    append_to_cfg_file $*
}

# Get a parameter value from the config file
# Parameters
#   Parameter to retrieve
# Relies on 
#   SMHA_CFG_FILE
# Returns
#   Sets the global variable CFG_RETURN_VALUE
#
get_value_from_cfg () {
    CFG_RETURN_VALUE=
    
    # Make sure the conf file exists
    if [ ! -f $SMHA_CFG_FILE ]; then
    	return 1
    fi
    
    #Set the search value
    local srch_option=\\-\\-${1}=
	local option_line=`grep "$srch_option" $SMHA_CFG_FILE`
#	debug "srch_option: $srch_option"
#	debug "option_line: $option_line"
	if [ -n "$option_line" ]; then
        # Get the value: delete everything up to the first = from the left
        CFG_RETURN_VALUE=${option_line#*=}
    else
        return 1            
	fi
	return 0
}

remove_option_from_cfg_file () {
	local opt=${1}
	
    # convert the variable name back to an option name
    local option_name=${opt//_/-}
    
    # if this is a multiple parm, then delete the __1 part
    option_name=${option_name%%--*}

    local sed_scr="/--$option_name.*/d;"

    # Create a temp file
    local tmpf=`mktemp /tmp/smha_rofcf.XXXXXX`  || fatal_error "`eval_gettext "Could not make temporary file"`"
    # Copy the contents to it
    cp $SMHA_CFG_FILE $tmpf    
    # Make the updates and write to the original
    debug "Removing $opt from config file"
    sed "$sed_scr" <$tmpf  >$SMHA_CFG_FILE        
    rm $tmpf                    
	
}

remove_from_cfg_file () {
	local opt
	
	for opt in $*
	do
	   remove_option_from_cfg_file $opt
	done
	
}

does_ha_config_exist () {
# Just check for a config file

    if [ -e "$SMHA_CFG_FILE" ]; then
        return 1
    else
        return 0
    fi

}

# Check if the TSA domain is already setup
# No parameters
# Relies on:
#   TSA_DOMAIN
# Returns
#   1 if the domain is setup
is_domain_setup () {	

	lsrpdomain -x $TSA_DOMAIN &> /dev/null	
	if [ $? -eq 0 ]; then
		return 1
	else
	    return 0
    fi
}

# Wait for the domain to come online
# No parameters
# Relies on:
#   TSA_DOMAIN
# Returns
#   0 if the domain is online, non-zero if the domain timed out or an error
wait_for_domain () {
	
	local i
	for i in {1..160}
    do
        local status=`LC_ALL=C lsrpdomain -x -l $TSA_DOMAIN 2> /dev/null | grep "OpState" | awk '{print $3}'`
        debug "wait_for_domain: status: $status"
        if [[ "$status" == "Online" ]]; then
            return 0
        else
            sleep 2
            # Stop this from looping if the domain doesn't exist.
            lsrpdomain -x -l $TSA_DOMAIN &> /dev/null
            if [ $? -eq 6 ]; then
                # TRANSLATORS: The "domain" specified in the message is the high availability domain, which 
                # is just the container that groups all the hardware and software resources that 
                # High Availability controls
            	fatal_error "`eval_gettext "The domain no longer exists.  Retry the operation, and if it fails again contact Software Support."`"
            fi
            sleep 2
        fi
    done
    return 1
}

# Start and wait for the domain to come online
# No parameters
# Relies on:
#   TSA_DOMAIN
# Returns
#   0 if the domain is online, non-zero if the domain timed out or an error
start_n_wait_for_domain () {

    local result=`startrpdomain $TSA_DOMAIN 2>&1`
    if [ $? -ne 0 ]; then
        # TRANSLATORS: The "domain" specified in the message is the high availability domain, which 
        # is just the container that groups all the hardware and software resources that 
        # High Availability controls
    	sm_msg "`eval_gettext \"Error starting the domain: \\\$result.  If this cannot be fixed automatically, further error messages will provide recovery instructions.\"`"
    fi
    debug "startrpdoman result:$result"
    
	sleep 10                
    wait_for_domain
    if [ $? -ne 0 ]; then
        # TRANSLATORS: The "domain" specified in the message is the high availability domain, which 
        # is just the container that groups all the hardware and software resources that 
        # High Availability controls
        fatal_error "`eval_gettext "Timed out waiting for TSA domain to come on line.  Retry the operation, and if it fails again contact Software Support."`"
    fi    
}

# Remove and wait for the domain to be deleted
# No parameters
# Relies on:
#   TSA_DOMAIN
# Returns
#   0 if the domain was deleted, non-zero if the remove timed out or an error
remove_n_wait_for_domain () {
	
    debug "Attempt to remove domain"
    local result=`rmrpdomain $TSA_DOMAIN 2>&1`
    if [ $? -ne 0 ]; then
    	debug "Error removing the domain: $result .  Going to try multiple more times in a loop."
    fi
    debug "$result"
                
    local i
    local rmdr
    for i in {1..160}
    do
    	debug "See if the domain has been removed."
        lsrpdomain -x $TSA_DOMAIN &> /dev/null
        local rc=$?  
        debug "lsrpdomain: rc: $rc"
        if [ $rc -eq 6 ]; then
            return 0
        else
            # It's still there, try to remove it again every so often
            let "rmdr = i % 5"
            if [ $rmdr -eq 0 ]; then
                # Make sure all the rg's are set to offline
                take_all_rg_offline            
                # Try deleting all the resources
                rm_all_rsrc
                sleep 2
            	rmrpdomain $TSA_DOMAIN
            fi
            
            sleep 5
        fi
    done

    return 1    
}

#is_rg_created () {
#	local rg=${1}
#	local found_rg=`lsrg | grep $rg | cut -d' ' -f1`
#	
#	debug "found rg:$found_rg:$rg"
#	if [ x"$found_rg" == x"$rg" ]; then
#		debug "is_rg_created: $rg =Y"
#		return 1 
#	fi
#	
#	debug "is_rg_created: $rg =N"
#	return 0
#}

is_rg_created () {
    local rg=${1}
    
    local i
    for i in {1..10}
    do
    	    
	    lsrg -g $rg &>/dev/null
	    local lsrg_rc=$?
	    
	    debug "found rg:$rg:$lsrg_rc"
	    case $lsrg_rc in
	    6 )
	        return 0
	    ;;
	    0 )
	        return 1
	    ;;
	    *)
	        sleep 5
	    ;;
	    esac    
    done
}



# Get the TSA rg status
# Parameters are:
#   rg name
# Return value:
#   the string value from TSA, like "online","offline", etc
get_rg_status ()
{    
    local rg=${1}

    rsrcstatus=`LC_ALL=C lsrg -A d -g $rg | grep "OpState" | awk '{print $3}'`

    GET_RSRC_STATUS=$rsrcstatus
    debug "rg status of $rg:$rsrcstatus"
    return 0       
}

# Get the TSA rg status
# Parameters are:
#   rg name
# Return value:
#   the OpState numeric value from TSA, like 0-6
# 0=unknown,1=online,2=offline,3=failed offline,4=stuck online,5=pending online,6=pending offline
get_rg_status_num ()
{    
    local rg=${1}

    rg_status=`LC_ALL=C lsrg -A d -g $rg OpState | grep "OpState" | cut -d' ' -f3`

    debug "rg status of $rg:$rg_status"
    return $rg_status       
}

    
wait_for_rg_offline () {
    local rg=${1}
    local i=
    
    is_rg_created $rg
    if [ $? -ne 1 ]; then
    	return 0
    fi
    
    for i in {1..160}
    do
        debug "See if $rg is offline"        
        get_rg_status_num $rg
        case $? in
        2|3 )
            return 0
        ;;
        4 )
            # stuck online, just fail
            return 1
        ;;
        *)
            sleep 5
        ;;
        esac
    done
    return 1
}


    
wait_for_mq_online () {
  
    local i=
    
    for i in {1..160}
    do
        debug "See if mq is online"        
        /etc/rc.d/smhamq status
        case $? in
        0 )
            return 0
        ;;
        *)
            # not online yet
            sleep 1
        ;;
        esac
    done
    return 1
}

wait_for_daemon_online () {
  
    local i=
    
    for i in {1..160}
    do
        debug "See if daemon is online"        
        /etc/rc.d/smhadm status
        case $? in
        0 )
            return 0
        ;;
        *)
            # not online yet
            sleep 1
        ;;
        esac
    done
    return 1
}
    
# Parameters are:
#   resource group name
take_rg_offline () {
    local rg=${1}
    
    is_rg_created $rg
    if [ $? -eq 1 ]; then
        #chrg -o offline ${rg}    
        debug "tsa call hook 2"
        call_tsa_chrg_o_l_u -o offline $rg
    fi
    
}

# Parameters are:
#   resource group name
take_rg_offline_n_wait () {
    local rg=${1}
    
    take_rg_offline ${rg}
    wait_for_rg_offline ${rg}
    if [ $? -ne 0 ]; then
        fatal_error "`eval_gettext \"Could not take resource group \\\$rg offline.  Retry the operation if it did not succeed.\"`"
    fi
    return 0
}

take_rgs_offline () {
	local rgs=$*
	
	local rg=
	for rg in $rgs
	do
		take_rg_offline_n_wait_nonfatal $rg
	done
}

# Parameters are:
#   resource group name
take_rg_offline_n_wait_nonfatal () {
    local rg=${1}
    
    take_rg_offline ${rg}
    wait_for_rg_offline ${rg}
   
    return $?
    
}

take_all_rg_offline () {
	

    local rglist=`LC_ALL=C lsrg Name | grep "Name" | cut -d' ' -f3`
    local rg
    for rg in $rglist
    do
        debug "Take $rg offline"
        take_rg_offline $rg        
    done    
	
}

rm_all_rg () {
    

    local rglist=`LC_ALL=C lsrg Name | grep "Name" | cut -d' ' -f3`
    local rg
    for rg in $rglist
    do
        debug "Remove $rg"
        rmrg $rg        
    done    
    
}


rm_all_rsrc () {
    local cmd_out=`LC_ALL=C rmrsrc -s "Name like '%'" IBM.Application Force=1 2>&1`
    debug "rm_all_rsrc:$cmd_out"
}

wait_all_rg_offline() {
	
	local we_done
    local statelist
    local state
    local i
        
    for i in {1..43}      # total wait time is (43 - 1) * 10 = 420 seconds or 7 minutes
    do   
        we_done=Y
        local statelist=$(lsrsrc-api -s IBM.ResourceGroup::::OpState 2> /dev/null)
	    for state in $statelist
	    do
	        debug "Take rg state is (${TSA_OFFLINE}=offline):$state"
	        case $state in 
	        	$TSA_OFFLINE | $TSA_FAILED_OFFLINE)
	        	    # Nothing to do... don't set the status - we_done=Y
	        	    echo " " > /dev/null
        	    ;;
	        	$TSA_UNKNOWN)
                    # If the state is unknown after a few iterations count it as offline.
	                if [ $i -le 2 ]; then
                        we_done=N
                    fi
        	    ;;    	
	        	*)
                    we_done=N
	        	;;
	        esac	        
        
	    done
	    
	    # If we didn't find any that weren't offline, the return
	    if [[ "$we_done" == "Y" ]]; then
	    	return
	    fi
	    # If this is the last loop then we timed out
        if [ $i -eq 43 ]; then
        	return 1
        fi
	        
	    sleep 10
    done
}

wait_for_rg_online () {
    local rg=${1}
    local i=
    for i in {1..160}
    do
        debug "See if $rg is online"        
        get_rg_status_num $rg
        case $? in
        1)
            return 0
        ;;
        *)
            sleep 5
        ;;
        esac
        
    done
    return 1
}

start_n_wait_for_rg_online () {
    local rg=${1}
    
    ch_rg_online ${1}   
    sleep 2
	wait_for_rg_online $rg
}

ch_rg_online () {
    local rg=${1}
    local rc
    
    # There are timing issues that can occur, so try changing the status and look for errors
    local i
    local cmd_out
    for i in {1..15}
    do
        debug "Change the rg $rg to online"        
        #local cmd_out=`chrg -o online $rg 2>&1`
        debug "tsa call hook 3"
        cmd_out=$(call_tsa_chrg_o_l_u -o online $rg)
        rc=$?
        case $rc in
        0)
            return 0
        ;;
        *)
            sleep 7
        ;;
        esac
        
    done
    debug "chrg output: $cmd_out"
    debug "Could not change nominal state of $rg to online. rc=$rc"
    return $rc
}

start_all_rg_online () {
    
#   local rglist=`LC_ALL=C lsrg Name | grep "Name" | cut -d' ' -f3`
    local rglist=$(LC_ALL=C lsrsrc-api -s IBM.ResourceGroup::'Owner != "Disabled"'::Name)
    local rg
    for rg in $rglist
    do
        debug "Set nominal state of $rg online"
        ch_rg_online $rg        
    done    
    
}


set_dir_ha_props () {
# Settings configured via <director>/data/TWGServer.prop
# HA.enabled = true|false
# HA.store.backup = true|false
# HA.events.shutdowndelay = 10

    local twgfile=$INSTALL_DIR/data/TWGServer.prop

	isset=`grep "HA.enabled" $twgfile | grep "true"`
	if [ -z "$isset"  ]; then
	    echo "HA.enabled = true" >>$twgfile
#	    echo "HA.store.backup = true" >>$twgfile
#	    echo "HA.events.shutdowndelay = $DIR_SHUTDOWN_DELAY" >>$twgfile
	fi

}

remove_dir_ha_props () {

    local twgfile=$INSTALL_DIR/data/TWGServer.prop

    perl -pi -e "s/^HA.enabled.*=.*//g" $twgfile
    
}

set_HA_enabled_for_activation () {
    local prop=com.ibm.director.HAEnabled
    if [ -f $HA_PROPS ]; then
        local prop_present=`grep ${prop} $HA_PROPS`
        if [ -n "$prop_present" ]; then
            perl -pi -e "s/^${prop}.*=.*/${prop}=true/g" $HA_PROPS
        else
            echo "${prop}=true" >> $HA_PROPS
        fi
    fi 
}

remove_HA_enabled_for_activation () {
    local prop=com.ibm.director.HAEnabled
    if [ -f $HA_PROPS ]; then
        local prop_present=`grep ${prop} $HA_PROPS`
        if [ -n "$prop_present" ]; then
            perl -pi -e "s/^${prop}.*=.*/${prop}=false/g" $HA_PROPS
        else
            echo "${prop}=false" >> $HA_PROPS
        fi
    fi 
}

# Parse a network alias string into pieces
# Parameters are:
#   network alias
# Return value:
#   Parsed pieces via the variables:
#     ALIAS_ADDR
#     ALIAS_MASK
#     ALIAS_NIC
parse_network_alias () {
	# Example: --network-alias=9.5.1.4,255.255.255.0,eth0
	#                          = Address,NetMask,NIC
	local float=${1}
    local float_sep=`echo $float | sed 's/,/ /g' `

	ALIAS_ADDR=`echo $float_sep | awk '{print $1}'`
	if [ -z "$ALIAS_ADDR" ]; then
	    debug "ALIAS_ADDR not set. $float" 
        # TRANSLATORS: The "network alias" $float specified in the message is something 
        # like 192.168.10.20,255.255.255.0,eth0 (so it is of the form 
        # <IP_ADDRESS,SUBNET_MASK,NETWORK_ADAPTER>), and is specified by the user
        # when they configure high availability
		fatal_error "`eval_gettext \"Could not parse network alias: \\\$float.  Check the parameters and try the command again.\"`"
	fi
	ALIAS_MASK=`echo $float_sep | awk '{print $2}'`
    if [ -z "$ALIAS_MASK" ]; then
        debug "ALIAS_MASK not set. $float" 
        fatal_error "`eval_gettext \"Could not parse network alias: \\\$float.  Check the parameters and try the command again.\"`"
    fi
	ALIAS_NIC=`echo $float_sep | awk '{print $3}'`
    if [ -z "$ALIAS_NIC" ]; then
        debug "ALIAS_NIC not set. $float" 
        fatal_error "`eval_gettext \"Could not parse network alias: \\\$float.  Check the parameters and try the command again.\"`"
    fi			
}

# Parse a network alias6 string into pieces
# Parameters are:
#   network alias6
# Return value:
#   Parsed pieces via the variables:
#     ALIAS_ADDR
#     ALIAS_PREFIX (Optional)
#     ALIAS_NIC
parse_network_alias6 () {
	# Example: --network-alias6=2001:db8::1428:57ab,64,eth0
	#                          = Address,NetPrefix,NIC
		
	local float=${1}
	
    ALIAS_NIC=${float##*,}
    local TEMP=${float#*,}
    ALIAS_PREFIX=${TEMP%%,*}
    ALIAS_ADDR=${float%%,*}
	
	if [ -z "$ALIAS_ADDR" ]; then
	    debug "ALIAS_ADDR not set. $float" 
		fatal_error "`eval_gettext \"Could not parse network alias: \\\$float.  Check the parameters and try the command again.\"`"
	fi
    if [ -z "$ALIAS_NIC" ]; then
        debug "ALIAS_NIC not set. $float" 
        fatal_error "`eval_gettext \"Could not parse network alias: \\\$float.  Check the parameters and try the command again.\"`"
    fi			
}

# This routine will return all IPv4 and IPv6 aliases (i.e. floaters) that are listed in the
# HA config file.  The addresses are set as a comma separated list in the following variables: 
#
#   ALIAS_IPV4_ADDRESSES <--- contains all IPV4 floaters
#   ALIAS_IPV6_ADDRESSES <--- contains all IPV6 floaters
#   ALIAS_ADDRESSES          <--- contains all IPV4 and IPV6 floaters
#
# For example, if the smhacfg.prop file contains: 
#   --network-alias6=2002:905:150e:151:9:5:39:147,64,eth0
#   --network-alias=9.5.39.147,255.255.252.0,eth0
#   --network-alias=192.168.1.147,255.255.255.0,eth1
#   This routine would return the following variable settings: 
#   ALIAS_IPV4_ADDRESSES=9.5.39.147,192.168.1.147
#   ALIAS_IPV6_ADDRESSES=2002:905:150e:151:9:5:37:147
#   ALIAS_ADDRESSES=9.5.37.147,192.168.1.147,2002:905:150e:151:9:5:39:147
#
get_all_alias_addresses () {
	
    ALIAS_IPV4_ADDRESSES=
    ALIAS_IPV6_ADDRESSES=
    ALIAS_ADDRESSES=

    if [ ! -e "${SMHA_CFG_FILE}" ]; then
        return;
    fi
   
    get_value_from_cfg agent-manager-address
    local agent_manager_address=${CFG_RETURN_VALUE}
    
    # We want the agent manager address first, so put that in now.
    ALIAS_ADDRESSES=${agent_manager_address}

    # Process IPv4 addresses first, they are stored as --network-alias=xxx

    local ipv4lines=`grep "network-alias=" ${SMHA_CFG_FILE}`
    if [ -n "$ipv4lines" ]; then
	for i in ${ipv4lines}
	do
            #echo "i is ${i}"
            if [ -n "$i" ]; then
                # We only parse the value on the right of the "="
                let ind=`expr index "${i}" =`
                alias_string="${i:$ind}"
                parse_network_alias "${alias_string}"
				
                if [ -z "${ALIAS_IPV4_ADDRESSES}" ]; then
		    # First address in the list, no preceding comma
		    ALIAS_IPV4_ADDRESSES="${ALIAS_ADDR}"
                else
		    # We already have set one, so need to add comma to separate
		    ALIAS_IPV4_ADDRESSES=${ALIAS_IPV4_ADDRESSES}","${ALIAS_ADDR}
                fi
                
                # If it isn't the agent manager address which is already in there, then we need to 
                # add it to the overall list
                if [[ "${agent_manager_address}" != "${ALIAS_ADDR}" ]]; then
                    ALIAS_ADDRESSES=${ALIAS_ADDRESSES}","${ALIAS_ADDR}
                fi
            fi
	done
    fi

    # Process IPv6 addresses next.  They are stored as --networ-alias6=xxxx
    local ipv6lines=`grep "network-alias6=" ${SMHA_CFG_FILE}`
    if [ -n "$ipv6lines" ]; then
        for i in ${ipv6lines}
        do
            #echo "i is ${i}"
            if [ -n "$i" ]; then
		# only parse the value on the right of the "="
		let ind=`expr index "${i}" =`
		alias_string="${i:$ind}"
		parse_network_alias6 "${alias_string}"
				
		if [ -z "${ALIAS_IPV6_ADDRESSES}" ]; then
                    # First address in the list, no preceding comma
                    ALIAS_IPV6_ADDRESSES=${ALIAS_ADDR}
                else
                    # We already have set one, so need to add comma to separate
                    ALIAS_IPV6_ADDRESSES=${ALIAS_IPV6_ADDRESSES}","${ALIAS_ADDR}
		fi

                # If it isn't the agent manager address which is already in there, then we need to 
                # add it to the overall list
                if [[ "${agent_manager_address}" != "${ALIAS_ADDR}" ]]; then
                    ALIAS_ADDRESSES=${ALIAS_ADDRESSES}","${ALIAS_ADDR}
                fi
            fi
        done
    fi

}

# Common routine that can collect the floating addresses and set them in 
# IPPreference.properties.  It assumes you have ensured that the file exists.
set_ip_preferences_list () {
    if [ -f $IPPREFERENCE_PROPS ]; then
        # Get all floater addresses - they will be in $ALIAS_ADDRESSES
        get_all_alias_addresses

        # If the property already exists in the file, just remove the entire line.  We will add it again at the end.
        # Doing it this way to avoid extreme escaping to replace possible backslashes in the existing value.
        local prop=com.ibm.director.server.preferred.ip
        perl -pi -e "s/^${prop}=.*\n//g" $IPPREFERENCE_PROPS
        # Escape any IPv6 colons in the IP address string (insert a backslash as needed)
        value=`echo ${ALIAS_ADDRESSES} | sed 's/:/\\\:/g'`
        # append the property to the end of the file
        echo "${prop}=${value}" >> $IPPREFERENCE_PROPS
    fi
}


# Returns 1=true, 0=false
is_mounted () {
	local mp=${1}    
    local foundmp=`grep " $mp " /proc/mounts | cut -d' ' -f2`
    if [ "x${foundmp}" == "x$mp" ]; then
    	debug "is_mounted: $mp found mounted: $foundmp"
        return 1
    else
        #debug "$mp is not mounted: ${foundmp} vs $mp"
        return 0
    fi
}

check_for_dependent_mirror () {    
	# Intent is to check if the mirror name passed in has another mirror
	# that is flagged as being dependent upon it.
	local mirror_to_check=${1}
    # Get the list of mirrors        
    tmpf=`mktemp /tmp/smha_uam.XXXXXX`  || fatal_error "`eval_gettext "Could not make temporary file"`"
    $GETMIRROR_CMD > $tmpf
    while read -r line
    do
        local mdev=`echo $line | cut -d' ' -f1` 
        local mmir=`echo $line | cut -d' ' -f2`
        local mlab=${mmir:1}
        # example: mdev=/dev/hda5, mmir=/mirror0, mlab=mirror0
        # check if this mirror has a dependency. 
        local dependency=`$GETMIRROR_DEPENDENCY_CMD -p $mmir`
        local dependent=$?
        if [ -z $dependency ]; then
            	debug "check_for_dependent_mirror: No dependency found for mount point $mmir"
        else 
        	# the mirror $mmir has a dependency on $dependency.
        	# Is $dependency the same name as $mirror_to_check?
        	if [ "$dependency" == "$mirror_to_check" ]; then
        		debug "check_for_dependent_mirror: $mmir is dependent on $mirror_to_check"
        		return 1
        	fi
        fi
    done < $tmpf    
    rm $tmpf
	return 0
}

mk_bkup_file_if_not_already () {
    local ffile=${1}
    local bkupfile=${2}
    	
    # Make a backup copy of the file if not there already
    if [ ! -f $bkupfile ]; then
        debug "Making backup of $ffile"
        cp -a $ffile $bkupfile
    fi

}


chk_nodes_opstate () {
    
    for opstate in `LC_ALL=C lsrsrc IBM.PeerNode OpState | grep OpState | cut -d' ' -f3`
    do
        if [ "$opstate" != "1" ]; then          
            # If the operational state is not 1 
            fatal_error "`eval_gettext "Passive node is not online, so a failover will not work at this time."`"
        fi          
    done
    
    
}

is_node_online () {
	
	local node=${1}
	local RC=0
    
    local opstate=`LC_ALL=C lsrsrc -s "Name='$node'" IBM.PeerNode OpState | grep OpState | cut -d' ' -f3`
    if [ "$opstate" == "1" ]; then
        RC=1    	          
    fi          
    debug "is_node_online: Node: $node Online: $RC"
    return $RC
    
}

is_director_online () {
    local RC=0
    
    local opstate=`LC_ALL=C lsrsrc -s "Name='$DIRECTOR_RSRC_NAME' && ResourceType=1" IBM.Application OpState | grep OpState | cut -d' ' -f3`
    if [ "$opstate" == "1" ]; then
        RC=1                  
    fi          
    debug "is_director_online: Online: $RC"
    return $RC
}




get_rsct_node_id () {
    local nodename=${1}

    local line=`grep $nodename $RSCT_NODEINFO_FILE`
    NODEID=${line##*0x}

}

set_rsct_node_id () {
    local nodeid=${1}
    
    if [ -e "$RSCT_NODEID_FILE" ]; then
        rm $RSCT_NODEID_FILE
    fi
    echo $nodeid >$RSCT_NODEID_FILE
    echo " " >>$RSCT_NODEID_FILE
    echo "# The first line of this file contains the RSCT node id of this" >>$RSCT_NODEID_FILE
    echo "# machine.  Please do not delete or modify it." >>$RSCT_NODEID_FILE
    
     /usr/sbin/rsct/install/bin/recfgct -s
    
}


is_address_ipv6 () {
	local address=${1}
	local ipv6=`echo $address | grep :`
	
	if [ -n "$ipv6" ]; then
	   return 1
	else
	   return 0
    fi
}

record_starting_termination () {
    local terminationFile=$SMHA_CONFIGDIR/terminating
    touch $terminationFile	
}

is_terminating () {
    local terminationFile=$SMHA_CONFIGDIR/terminating
    
    if [ -e "$terminationFile" ]; then    	
       return 1
    else
       return 0
    fi
	
}

is_configured () {
    local RC=0  
    if [ -f $SMHA_CFG_FILE ]; then
        # It's there but is the node parameter present?
        get_value_from_cfg node
        if [ $? -eq 0 ]; then
        	# And the domain seems to be setup?
        	is_domain_setup
            if [ $? -eq 1 ]; then       
                RC=1
            fi
        fi
    fi

    return $RC            
}

# check to see if HA is configured without checking the peer domain status

is_HAconfigured () {
    local RC=0  
    if [ -f $SMHA_CFG_FILE ]; then
        # It's there but is the node parameter present?
        get_value_from_cfg node
        if [ $? -eq 0 ]; then
            RC=1
        fi
    fi

    return $RC            
}


retry_cmd () {
    local times=$1
    local expect_rc=$2
    shift 2
    local run_cmd="$*"

    debug "retry_cmd: times:$times rc:$expect_rc Command: $run_cmd"         
   
    local i
    for i in $(seq 1 ${times})
    do 
        debug "Retry Command: $run_cmd"         
        local cmd_out=`$run_cmd 2>&1`
        rc=$?
        debug "Retry: expect_rc=$expect_rc, rc=$rc"
        debug "Retry cmd_out:$cmd_out"
        if [[ "$rc" == "$expect_rc" ]]; then
            return $rc
        else
           debug "retry_cmd:$run_cmd"
           debug "output:$cmd_out" 
           sleep 2     
        fi
    done
    
}

make_relationship () {
    local RC

    smha_make_relationship $@
    RC=$?

    return $RC
}

#make_rsrc () {
#    local name=${1}
#    local type=${2}
#    local cmd=${3}
#    
#    local i
#    for i in {1..100}
#    do
#    
#        cmd_out=`$cmd 2>&1`
#        
#        rsrc_exists ${name} ${type}
#        if [ $? -eq 1 ]; then
#            debug "Made rsrc $name $type"
#            
#            return 0
#        else
#            debug "Retry making rsrc $name $type"
#            sleep 5       
#        fi
#    done
#    return 1
#}

rsrc_exists () {
    local name=${1}
    local type=${2}
    
    present=`LC_ALL=C lsrsrc -s "Name = '$name'" $type | grep ResourceType `
    if [ -n "$present" ]; then
        return 1
    else
        return 0
    fi 
        
}

add_to_rg () {
    local RC

    smha_add_to_rg $@
    RC=$?

    return $RC
}

chrg_l_none () {
    local rg=${1}

    local i
    for i in {1..100}
    do

        #local cmd_out=`chrg -l None $rg 2>&1`
        debug "tsa call hook 4"
        local cmd_out
        cmd_out=$(call_tsa_chrg_o_l_u -l None $rg)
        local cmd_rc=$?        
        if [ $cmd_rc -eq 0 ]; then
            debug "chrg -l None $rg -worked."
            
            return 0
        else
            debug "Failed:chrg -l None $rg :: rc:$cmd_rc output:$cmd_out"
            sleep 5       
        fi
    done
    return 1
}


get_ipv4_nic_address () {
    local nic=${1}
    #inet addr:9.10.111.244  Bcast:9.10.111.255  Mask:255.255.255.0
    ADDRESS=`LC_ALL=C ifconfig $nic | grep "inet addr" | awk '{print $2}' | cut -d: -f2`    
}

get_ipv6_nic_address () {
    local nic=${1}
    #inet6 addr: fe80::221:86ff:fe1a:3e1e/64 Scope:Link
    ADDRESS=`LC_ALL=C ifconfig $nic | grep "inet6 addr" | awk '{print $3}' | cut -d/ -f1`    
}


log_command_success () {
    
     log_command success
}

log_command_failed () {
    
     log_command failed
}


log_command () {
    
#     /opt/ibm/director/vmi/bin/logevent -t success | failed -c command name [-a command argument    
     /opt/ibm/director/vmi/bin/logevent -t $1 -c $EXTPROGNAME -a "$ORIG_PARMS"
     
}

is_automation_on() {
	
    local is_on=`LC_ALL=C lssamctrl Automation | grep Automation | awk '{print $3}'`
    if [ $? -eq 0 ]; then
        return $is_on
    else
        # Command didn't complete successfully
        return 99
    fi 
	
}

turn_on_automation () {
    #local result=`samctrl -M F 2>&1`
    debug "tsa call hook 5"
    local result
    result=$(call_tsa_samctrl_M_r F)
    debug "turn on automation: $result"
}

turn_off_automation () {
    #local result=`samctrl -M T 2>&1`
    debug "tsa call hook 6"
    local result
    result=$(call_tsa_samctrl_M_r T)
    debug "turn on automation: $result"
}

create_automation_flagfile () {
    touch $AUTOMATION_FLAGFILE
    
}

delete_automation_flagfile () {
    rm -f $AUTOMATION_FLAGFILE
    
}

add_ha_to_inittab () {
	
	# add the line if it doesn't exist
	local hainittab=`grep sha /etc/inittab | grep $SMHA_DIR`
	if [ -z "$hainittab" ]; then 
	   echo "sha:35:once:/opt/ibm/director/ha/bin/priv/smhamaint.sh #HA" >> /etc/inittab
	fi
	
	# uncomment it if it happens to exist
	uncomment_out_ha_inittab
}

comment_out_ha_inittab () {
	    perl -pi -e "s/^sha:35/#sha:35/g" /etc/inittab	
    
}
uncomment_out_ha_inittab () {
        perl -pi -e "s/^#sha:35/sha:35/g" /etc/inittab    
}

set_single_node_mode () {
    debug "setting single node mode"
    
    get_local_rsct_node_name
    local rsct_node=$LOCAL_RSCT_NODE_NAME
    
    chrsrc_out=`LC_ALL=C chrsrc -s "Name like '%' && ResourceType=1" IBM.Application NodeNameList="{'$rsct_node'}" 2>&1`
    debug "chrsrc out: $chrsrc_out"
}

set_dual_node_mode () {
    debug "setting dual node mode"
    
    get_value_from_cfg node-order
    node_order=$CFG_RETURN_VALUE    
    #debug "node_order is:$node_order"
    #hosts_list=`echo "'$node_order'" | sed "s/,/','/g"`
    
    get_local_rsct_node_name
    local rsct_node=$LOCAL_RSCT_NODE_NAME
    get_remote_rsct_node_name
    local remote_rsct_node=$REMOTE_RSCT_NODE_NAME
    
    if [[ "$host_list" == ${rsct_node},* ]]; then
    	hosts_list="'$rsct_node','$remote_rsct_node'"
    else
        hosts_list="'$remote_rsct_node','$rsct_node'"
    fi
    
    debug "hostlist is:$hosts_list"
    chrsrc_out=`LC_ALL=C chrsrc -s "Name like '%' && ResourceType=1" IBM.Application NodeNameList="{$hosts_list}" 2>&1`
    debug "chrsrc out: $chrsrc_out"
}

is_single_node_mode () {
    
    local dual_mode=`LC_ALL=C lsrsrc -s "Name like 'director' && ResourceType=1" IBM.Application NodeNameList | grep NodeNameList | grep ,`
    if [ -z "$dual_mode" ]; then
        return 1
    else
        return 0
    fi
}



create_exclude_peer_flagfile () {
    touch $EXCLUDE_PEER_FLAGFILE
    
}

delete_exclude_peer_flagfile () {
	debug "delete exclude node flag" 
    rm -f $EXCLUDE_PEER_FLAGFILE
    
}

FLOAT_RGS="$STORAGE_RG $DB2_RG $DIRECTOR_RG"
exclude_node_from_float_rgs () {
    local ex_node=${1}
    local ex_action=${2}
    
    local list_of_rgs=$FLOAT_RGS
    # Add the dhcp rg if it's shared configuration
    get_value_from_cfg dhcp
    case $CFG_RETURN_VALUE in
    "shared" )
        list_of_rgs="$FLOAT_RGS $DHCP_RG"
    ;;
    esac    
    
    local rg
    for rg in $list_of_rgs
    do
        debug "$ex_action $ex_node rg: $rg"
        
        is_rg_created $rg
        if [ $? -eq 1 ]; then
        
            lock_rg $rg
            	        
	        local max_tries=25
	        local count=1
	        local chrg_rc
	        while [ $count -le $max_tries ]
	        do
		        if [[ "$ex_action" == "exclude" ]]; then
		            #chrg_out=`chrg -u a -x $ex_node $rg 2>&1`
                            debug "tsa call hook 7"
		            chrg_out=$(call_tsa_chrg_o_l_u -u a -x $ex_node $rg)
		        else
		            #chrg_out=`chrg -u d -x $ex_node $rg 2>&1`
                            debug "tsa call hook 8"
		            chrg_out=$(call_tsa_chrg_o_l_u -u d -x $ex_node $rg)
		        fi
		        chrg_rc=$?
		        debug "Try $count chrg rc: $chrg_rc out: $chrg_out"
		        if [ $chrg_rc -eq 0 ]; then
		        	break
		        fi
		        # If it wasn't excluded in the first place, skip it.
                if [ $chrg_rc -eq 6 ]; then
                    if [[ "$ex_action" == "include" ]]; then
                        break
                    fi
                fi
		        
                let "count = count + 1"
	        
            done
            unlock_rg $rg
        else
            debug "RG $rg not excluded since it doesn't exist."
        fi
                
    done
    
    local exc_list=`LC_ALL=C lsrg -s "Name like '%-rg'" | grep -e Excl -e Name`
    debug "Excluded List: $exc_list"
    
    
}

lock_rg () {
	local rg=${1}
    
    local lock_rc=
    local rgreq_out=
        
    local max_tries=25
    local count=1
    local chrg_rc
    while [ $count -le $max_tries ]
    do    
	    rgreq_out=`LC_ALL=C rgreq -o lock $rg 2>&1`
	    lock_rc=$?
	    debug "rgreq rc: $lock_rc out: $rgreq_out"
	    
	    local lock_success=`echo $rgreq_out | grep "returned Token"`
	    if [ -n "$lock_success" ]; then
	    	break;
	    fi
        debug "Lock attempt on $rg failed. $lock_success"
        let "count = count + 1"    
    done
}

unlock_rg () {
    local rg=${1}
    local lock_rc=
    local rgreq_out=
    
    local max_tries=25
    local count=1
    local chrg_rc
    while [ $count -le $max_tries ]
    do
	    rgreq_out=`LC_ALL=C rgreq -o unlock $rg 2>&1`
	    lock_rc=$?
	    debug "rgreq rc: $lock_rc out: $rgreq_out"
        local lock_success=`echo $rgreq_out | grep "returned Token"`
        if [ -n "$lock_success" ]; then
            break;
        fi
        debug "Unlock attempt on $rg failed. $lock_success"
        let "count = count + 1"
    done
	
}

is_excluded_list_empty () {
	
	local lsrg_out=`LC_ALL=C lsrg -s "Name like '%'" | grep ExcludedList | awk '{print $3}' | grep -v {}`
	debug "lsrg out:$lsrg_out"
	if [ -z "$lsrg_out" ]; then
		return 1
	else
	    return 0
	fi
}

exclude_peer_node () {
    get_remote_rsct_node_name    
    exclude_node_from_float_rgs $REMOTE_RSCT_NODE_NAME exclude   
}

include_peer_node () {
    get_remote_rsct_node_name
    exclude_node_from_float_rgs $REMOTE_RSCT_NODE_NAME include   
}

include_this_node () {
    get_local_rsct_node_name
    exclude_node_from_float_rgs $LOCAL_RSCT_NODE_NAME include   
}

include_all_nodes () {
    include_peer_node
    include_this_node   
}

exclude_node_from_domain () {
    get_local_rsct_node_name
    local rsct_node=$LOCAL_RSCT_NODE_NAME
    
	domain_excluded_node $rsct_node exclude
}
include_node_in_domain () {
	get_local_rsct_node_name
    local rsct_node=$LOCAL_RSCT_NODE_NAME
	
	domain_excluded_node $rsct_node include
}

domain_excluded_node () {
    local ex_node=${1}
    local ex_action=${2}
    
    debug "domain_excluded_node node: $ex_node action: $ex_action"        
        
    
    local max_tries=25
    local count=1
    while [ $count -le $max_tries ]
    do
        if [[ "$ex_action" == "exclude" ]]; then
            sam_out=`LC_ALL=C samctrl -u a $ex_node 2>&1`
        else
            sam_out=`LC_ALL=C samctrl -u d $ex_node 2>&1`
        fi
        local sam_rc=$?
        debug "Try $count samctrl rc: $sam_rc out: $sam_out"
        # See if the command worked by looking at the exlude list        
        local node_present=`LC_ALL=C lssamctrl 2>&1 | grep ExcludedNodes | grep $ex_node`
        local it_worked=0
        
        if [[ "$ex_action" == "exclude" ]]; then
        	if [ -n "$node_present" ]; then
        		it_worked=1
        	fi
        else
            if [ -z "$node_present" ]; then
                it_worked=1
            fi
        fi

        if [ $it_worked -eq 1 ]; then
            break
        else
          let "count = count + 1"
          sleep 4
        fi
    
    done
    
    local exc_list=`LC_ALL=C lssamctrl 2>&1`
    debug "samctrl Excluded List: $exc_list"
	
}

create_shutdown_halt_flagfile () {
    touch $SHUTDOWN_HALT_FLAGFILE    
}
delete_shutdown_halt_flagfile () {
    rm -f $SHUTDOWN_HALT_FLAGFILE    
}

create_stop_maint_flagfile () {
    touch $STOP_MAINT_FLAGFILE    
}
delete_stop_maint_flagfile () {
    rm -f $STOP_MAINT_FLAGFILE    
}

handle_shutdown_halt_flagfile () {
   if [ -e $SHUTDOWN_HALT_FLAGFILE ]; then
        debug "handle_shutdown_halt_flagfile: found halt flag file. Processing it"
        
        include_node_in_domain
        delete_shutdown_halt_flagfile
    fi
}

handle_exclude_peer_flagfile () {
   if [ -e $EXCLUDE_PEER_FLAGFILE ]; then
        debug "handle_exclude_peer_flagfile: found exclude peer flag file - processing it"
        
        # Wait enough time to get the resources starting on this node
        # ...as long as the other node didn't exclude us...
        
        were_exc=`LC_ALL=C lsrg -s "Name like '%-rg'" 2>&1 | grep ExcludedList | grep $HOST`
        if [ -z "$were_exc" ]; then
            wait_for_rg_online $STORAGE_RG
        else
            debug "handle_exclude_peer_flagfile: Found ourselves being excluded by peer."
        fi
        
        include_peer_node
       
        delete_exclude_peer_flagfile
    fi
}


set_crit_rsrc_protection () {
	local level=$1
    local chrsrc_out=`LC_ALL=C chrsrc -c IBM.PeerNode CritRsrcProtMethod=$level 2>&1`
    debug "chrsrc -c IBM.PeerNode CritRsrcProtMethod=$level output:$chrsrc_out"
}

create_syncconf_flagfile () {
    touch $SYNCCONFIG_FLAGFILE
    
}

delete_syncconf_flagfile () {
    rm -f $SYNCCONFIG_FLAGFILE
    
}

create_refresh_managementIP_flagfile () {
    touch $REFRESH_MANAGEMENTIP_FLAGFILE
    
}

delete_refresh_managementIP_flagfile () {
    rm -f $REFRESH_MANAGEMENTIP_FLAGFILE
    
}

get_node_value () {
    get_value_from_cfg node
    node=$CFG_RETURN_VALUE                            
}

sync_tsa_config () {
    
    # get the node value from the config file.  This will ensure it's the latest
    get_node_value
        
    local cmd_out=`LC_ALL=C runact -c IBM.PeerDomain SyncConfig DomainName=$TSA_DOMAIN TargetNodes="{'$HOST','$node'}"`
    debug "SyncConfig output:$cmd_out"
}


add_etc_hosts_entry () {
	local address=${1}
	local hname=${2}
	 
	echo "$address $hname" >> /etc/hosts
	debug "add /etc/hosts entry: $address $hname"
	
}

ping_host () {
	local other_node=$1
	
	ping -c 5 -q $other_node 1> /dev/null 2>&1  
    RC=$?
    debug "ping $other_node rc:$RC"
    if [ $RC -ne 0 ]; then
        # since we can't do an IPv4 ping, try IPv6 ping
        ping6 -c 5 -q $other_node 1> /dev/nul 2>&1
        RC=$?
        debug "ping6 $other_node rc:$RC"
        
    fi
    
    return $RC
	
}

# I see that there was a get_host_address routine in smhaagent.sh  that checked DNS first before
# checking /etc/hosts.  I don't understand that but it is too late in the release for me to 
# feel comfortable changing that.  This is sort of the same routine.  It lets "ping" find the address
# and ping uses /etc/hosts first, and then DNS next...

get_host_address_etc_hosts_override () {
    local name=${1}
        
    HOST_ADDRESS=

    # PING pfeecl3lp5.in.ibm.com (9.126.134.214) 56(84) bytes of data.
    
    local pcmd=`ping -c 1 $name | grep PING`
    debug "get_host_address_etc_hosts_override: ping output: $pcmd"
    local h1=${pcmd#*(}
    local h2=${h1%%)*}
    debug "h2:$h2"
    HOST_ADDRESS=$h2        
       
 	debug "get_host_address_etc_hosts_override: $name $HOST_ADDRESS"
 	
    if [ -z "$HOST_ADDRESS" ]; then
        fatal_error "`eval_gettext \"Could not resolve address for host name \\\$name.  Make sure that the host name is in the DNS and that this system can connect to the DNS, or that the host name is in the hosts file.\"`"
    fi
    
}

get_admin_id () {
	local get_admin=/opt/ibm/director/vmi/bin/lsdefaultadmin
	
	if [ -n "$ADMIN_ID" ]; then
		return
	fi 
	
	if [ -x "$get_admin" ]; then
		local admin_id=`$get_admin`
	else
	    local admin_id=sysadmin
	fi
	
	ADMIN_ID=$admin_id
	
}

get_cfg_user_id () {
	
    if [ -n "$CFG_USER_ID" ]; then
        return
    else
        if [ -e $SMHA_CONFIGDIR/.cfg_user ]; then
        	CFG_USER_ID=`cat $SMHA_CONFIGDIR/.cfg_user`
        else
            get_admin_id
            CFG_USER_ID=$ADMIN_ID
        fi
    fi     
}

save_cfg_user_id () {
	local user_id=$1
	
	echo "$user_id" > $SMHA_CONFIGDIR/.cfg_user
	CFG_USER_ID=$user_id
}

rm_cfg_user_file () {
	rm -f $SMHA_CONFIGDIR/.cfg_user
}

# Use this method to record the HA configuration status so that smhastatus can 
# report it as a translated string.  If this status is a fatal error that will
# immediately stop the HA config, use record_cfg_status_fatal_error instead
record_cfg_status () {
    # NOTE: the status parameter should NOT be translated, because it needs to be
    #       stored in the CFG_STATUS_FILE as a key, so when retrieved they can be 
    #       translated at that time.  Note that the status parameter DOES need to 
    #       show up in a script as a parameter to eval_gettext at some point so 
    #       the string will be put in the .po files when they are auto generated.
    #       One good way to do this is to use the following two lines to record a 
    #       status line for the user to see.
    #
    #       sm_msg "`eval_gettext "Show this line as status to the user."`"
    #       record_cfg_status "Show this line as status to the user."
    #
    #       The sm_msg call puts the translated version of "Show this line..."
    #       into the system log, the smha.log, and sends it to stdout, but more 
    #       importantly marks that string to be put into the .po file so it can be 
    #       translated into other languages
    #
    #       The record_cfg_status call of course calls this method, which puts the
    #       string (untranslated) into the CFG_STATUS_FILE.  Then if the user runs
    #       smhastatus, that untranslated string will be retrieved and translated
    #       into whatever locale the user's ssh session is in and displayed to the 
    #       user.
    
	local status=$*
	
	# record the most recent status
	echo "$status" >> $CFG_STATUS_FILE

	# Send the MQ message to the secondary node to record the status on that node
	run_init "status" "$status"
}

# Use this method to record the HA configuration status so that smhastatus can 
# report it as a translated string.  Only us this method if the status is a
# fatal error that will immediately stop the HA config.  Otherwise use 
# record_cfg_status instead
record_cfg_status_fatal_error () {
    # see the NOTE above in record_cfg_status, they apply for this method also
    
	local status=$*
	
	# record the status and touch the flag file to indicate a fatal error occurred
	echo "$status" >> $CFG_STATUS_FILE
	touch $ERROR_DURING_CONFIG_FLAGFILE
	
	revert_cas_ip_property restartAgent
	
	# Send the MQ message to the secondary node to record the status and flag file
	# on the secondary node
	run_init "status" "-E" "$status"
}

rm_cfg_status_file () {
	rm -f $CFG_STATUS_FILE
}

is_passive () {
    smha_is_passive
    return $?
}

get_local_rsct_node_name () {

    if [ -z "$LOCAL_RSCT_NODE_NAME" ]; then
       LOCAL_RSCT_NODE_NAME=$(smha_get_local_rsct_node_name)
       debug "LOCAL_RSCT_NODE_NAME:$LOCAL_RSCT_NODE_NAME"
    fi
}

get_remote_rsct_node_name () {

    if [ -z "$REMOTE_RSCT_NODE_NAME" ]; then
        REMOTE_RSCT_NODE_NAME=$(smha_get_remote_rsct_node_name)
        debug "REMOTE_RSCT_NODE_NAME:$REMOTE_RSCT_NODE_NAME"
    fi
}

# N.B. it is assumed that a mutex is held only briefly (less than a second or two). For longer
# helds locks, use rw_lock.

MUTEX_DIR=$SMHA_CONFIGDIR/.mutex
PROCID4LOCKS=
mutex_on () {
	local lock=${1}
    local sleep_timer="0.2"
	local lockflag=MUTEX_${lock}
	local mutex_dir=${MUTEX_DIR}/${lock}
    local start_cleanup=0

    if [ -z "$PROCID4LOCKS" ]; then
        # this variable only needs to be set once in the process
        PROCID4LOCKS="$$.$(ps --no-headers -p $$ -o start)"
    fi
	
	mkdir -p `dirname $mutex_dir`  
    #debug "mutex $lock on"
    if [ -z "${!lockflag}" ]; then
        while true; do
            mkdir $mutex_dir 1>/dev/null 2>&1
            mkdir_rc=$?
            if [ $mkdir_rc -eq 0 ]; then
                touch ${mutex_dir}/${PROCID4LOCKS}     # denotes ownership of mutex
            	eval ${lockflag}=on
                #debug "mutex $lock successfully locked, owner is ${PROCID4LOCKS}"
                return
            else
                #debug "mutex $lock already locked, sleeping. rc=$mkdir_rc"
                if (( ++start_cleanup == 50 )); then
                    /opt/ibm/director/ha/bin/priv/smhamutexclean ${mutex_dir} &> /dev/null &
                fi
                sleep $sleep_timer
            fi
            
        done
    fi
}

mutex_off () {
    local lock=${1}
    local lockflag=MUTEX_${lock}
    local mutex_dir=${MUTEX_DIR}/${lock}
	
    if [ -n "${!lockflag}" ]; then
        #debug "mutex $lock off"     
        rm -rf $mutex_dir
    fi
    eval ${lockflag}=
}

mutex_clear_all () {
	if [ -d $MUTEX_DIR ]; then
	   rm -rf $MUTEX_DIR
	fi
}

# Implement a "read/write" lock, wherein multiple processes can hold the lock for
# "read" with no writer, or only one process can hold the lock for "write", with no
# readers. If the caller can successfully get the lock, the function returns 0, otherwise 1.
# This function also removes "stale" lock files, i.e. files whose specified process owner no
# longer exists. The implication is that calling processes do not have to clean-up the lock
# if they terminate without freeing the lock first.
#
# N.B. this implementation supports ONLY the acquisition of a single read lock or a single
# write lock (named the same), but not both, in the process at any point in time.
#
# Arg 1:  command (read or write)
# Arg 2:  operation (get or free)
# Arg 3:  name of lock
# Arg 4:  name of resource requesting lock (used to associate a resource to a lock)
# Arg 5:  time (in seconds) to wait for the lock (only for get)

rw_lock () {
    local rc=0
    local cmd=${1}
    local op=${2}
    local lock=${3}
    local rsrc=${4}
    local timer=${5}
    local RWDIR=${MUTEX_DIR}/${lock}RW
    local procID
    local pstart
    local pid
    local lockfile
    local stalelock
    local lsout
    local lsarg

    if [ -z "$PROCID4LOCKS" ]; then
        # this variable only needs to be set once in the process
        PROCID4LOCKS="$$.$(ps --no-headers -p $$ -o start)"
    fi

    if [ $cmd = "read" ]; then
        lockfile="reader_${rsrc}_${PROCID4LOCKS}"
        lsarg=${RWDIR}/writer    # look for writer
    else
        lockfile="writer_${rsrc}_${PROCID4LOCKS}"
        lsarg=${RWDIR}/           # look for readers or writer
    fi

    mutex_on $lock
    if [ "$op" = "get" ]; then
        if [ ! -d $RWDIR ]; then
            mkdir $RWDIR
        fi

        for (( ++timer; timer > 0; timer-- )); do
            # first, check for stale locks, i.e. locks where the process no longer exists

            for stalelock in $(ls $RWDIR)
            do
                procID=${stalelock##*_}      # get proc ID, i.e. PID + start time
                pid=${procID%%.*}            # get PID from procID

                pstart="$(ps --no-headers -p $pid -o start 2> /dev/null)"

                if [ -z "$pstart" -o "$procID" != "${pid}.${pstart}" ]; then
                    debug "rw_lock_${cmd} removing stale lockfile $stalelock"
                    rm -f ${RWDIR}/$stalelock
                fi
            done

            lsout="$(ls ${lsarg}* 2> /dev/null)"
            if [ -z "$lsout" ]; then
                touch ${RWDIR}/${lockfile}
                break
            fi

            if (( timer == 1 )); then
                rc=1
            else
                debug "rw_lock_${cmd} waiting for $lock"

                mutex_off $lock
                sleep 1
                mutex_on $lock
            fi
        done
    else
        rm -f ${RWDIR}/${lockfile}
    fi
    mutex_off $lock

    debug "rw_lock_${cmd} $op ${lock}RW/${lockfile}, rc=$rc"
    return $rc
}

#
# This function waits for the specified OpState change of the 
# peer node IBM.PeerNode resource.
# It will wait for up to 4 hours, sleeping for 10 second intervals.
#
# Parm1: The desired state.  
# 	Value: online
#	Result: wait for OpState of IBM.PeerNode resource for peer node to go to online
# 
#	Value: offline
# 	Result: wait for Opstate of IBM.PeerNode resource for peer node to go to 
#			offline	or failed offline	
#  

wait_for_peer_state () {
	local RC=0
	local desired_state=$1
	local done_sleeping=0
	debug "wait_for_peer_state: entered with $desired_state"

	get_remote_rsct_node_name
	node=$REMOTE_RSCT_NODE_NAME
	local i
	# wait for up to 4 hours
	for i in {1..1440}
	do
		cmd_out=`LC_ALL=C lsrsrc-api -s "IBM.PeerNode::Name == \"$node\"::OpState"`
		RC=$?
		if [ $RC -eq 0 ]; then
			# command worked 
			if [[ "$desired_state" == "online" && "$cmd_out" == "1" ]]; then
				debug "wait_for_peer_state: detected peer node $node is online"
				done_sleeping=1
			elif [[ "$desired_state" == "offline" ]] &&  [[ "$cmd_out" == "2" || "$cmd_out" == "3" ]] ; then
				# the resource is offline or failed offline
				debug "wait_for_peer_state: detected peer node $node is offline or failed offline"
				done_sleeping=1
			fi
		fi
		if [ $done_sleeping -eq 1 ]; then
			break
		else
			debug "wait_for_peer_state: sleep number $i for 10 seconds, desired_state=$desired_state, actual_state=$cmd_out"
			sleep 10
		fi
	done

	if [ $done_sleeping -ne 1 ]; then
		debug "wait_for_peer_state: timed out waiting for node $node to obtain state $desired_state"
		RC=4
	fi
	debug "wait_for_peer_state: RC=$RC"
	return $RC		
}

#
# This function waits for the peer node's "DRBD base resource" (drbdb) OpState
# to go online.
#
# It will wait for up to 4 hours, sleeping for 10 second intervals.
#
wait_for_peer_drbd_base() {
	local RC=0
	debug "wait_for_peer_drbd_base: entered"

	get_remote_rsct_node_name
	node=$REMOTE_RSCT_NODE_NAME
	local i
	# wait for up to 4 hours
	for i in {1..1440}
	do
		cmd_out=`LC_ALL=C lsrsrc-api -s "IBM.Application::Name  == \"drbdb\" && NodeNameList=\"$node\"::OpState"`
		RC=$?
		if [ $RC -eq 0 ]; then
			# command worked 
			if [[ "$cmd_out" == "1" ]]; then
				debug "wait_for_drbd_base: detected peer node $node drdb base resource coming online"
				done_sleeping=1
				break
			else
				debug "wait_for_peer_drbd_base: sleep number $i for 10 seconds, desired_state=1, actual_state=$cmd_out"
				sleep 10
			fi
		else
			debug "wait_for_peer_drbd_base: lsrsrc-api failed with RC=$RC"
			sleep 10
		fi
	done
	
	if [ $done_sleeping -ne 1 ]; then
		debug "wait_for_peer_drbd_base: timed out waiting for node $node drbd base resource to come online"
		RC=1
	fi
	debug "wait_for_peer_drbd_base: RC=$RC"
	return $RC		
}


# the following function checks the state of the director peer domain
#
# Exit code: 0 - PD is online
#            1 - PD is down (offline, stopped)
#            2 - PD state is pending online or offline
#            3 - PD not configured
#            9 - error

check_PD_status () {
    local RC=9          # assume bad, VERY BAD
    local stat_RC
    local Status=Error

    OPSTATE=$(CT_MANAGEMENT_SCOPE=1 lsrsrc-api -s IBM.PeerDomain::'Name == "director"'::OpState 2> /dev/null)
    stat_RC=$?

    if [ $stat_RC -eq 0 ]
    then
        debug "Peer Domain OpState = $OPSTATE"
        case $OPSTATE in
            1) RC=0 ; Status=online ;;
            2) RC=1 ; Status=offline ;;
            5) RC=2 ; Status="pending online" ;;
            6) RC=2 ; Status="pending offline" ;;
        esac
    elif [ $stat_RC -eq 6 ]       # no resources found
    then
        RC=3                 # PD is not configured
        Status="not configured"
    elif [ $stat_RC -eq 5 ]       # the RMC daemon is down
    then
        RC=1
        Status="RMC is down"
    fi

    debug "Peer Domain status: $Status"

    return $RC
}

# the following function checks if the frmstandby-rg, which has been found
# to be in a failed offline state, can be ignored. First, check if it meets
# the following two conditions (test A)
#
#  1) the local constituent resource of the frmstandby-rg's managed resource is Offline
#  2) the peer node is offline
#
# If these conditions are not met, then check the Director resource group's OpState.
# If the Director RG is online, then the failed offline state of the frmstandby-rg
# is temporary and can be ignored (test B).
#
# Arg 1: the OpState of the Director resource group
#
# Exit code: 0 - don't ignore the state of the frmstandby-rg
#            1 - ignore the state of the frmstandby-rg; it's failed offline due
#                to the peer node being down
#            2 - ignore the state of the frmstandby-rg; temporary condition

ignore_frmstandby_rg () {
    local DirOpState=$1
    local RNisOnline
    local RC=0

    get_local_rsct_node_name      # this sets LOCAL_RSCT_NODE_NAME variable
    get_remote_rsct_node_name     # this sets REMOTE_RSCT_NODE_NAME variable
    is_node_online $REMOTE_RSCT_NODE_NAME     # returns 1 if node is online
    RNisOnline=$?                             # a value of 1 is "online"


    if OpState=$(lsrsrc-api -s "IBM.Application::Name == 'frmstandby' && NodeNameList == {'${LOCAL_RSCT_NODE_NAME}'}::OpState" 2> /dev/null); then

        if [ $OpState -eq 2 -a $RNisOnline -ne 1 ]; then
            debug "frmstandby local constituent is offline and peer node is down"
            RC=1
        elif [ $DirOpState -eq 1 ]; then
            debug "$SMHA_DIRECTOR_RG is online; frmstandby-rg failed offline is temporary"
            RC=2
        fi
    fi

    debug "ignore_frmstandby_rg returning RC=$RC ... DirOpState=$DirOpState"

    return $RC
}

# the following function checks the state of the resource groups. This
# function is only called if the local node is Passive. The local node may be
# passive due to the other node being Active OR the resource groups are offline.
# The OpState of the RGs are also verified to ensure that the groups are not in
# transition.
#
# Exit code: 0 - all resource groups are online
#            1 - all resource groups are offline
#            2 - resource groups are changing state, i.e. pending online/offline
#            3 - all RGs match their OpState to Nominal state (but not one of the above)
#            9 - error

check_rgs () {
    local RC
    local OpState
    local NominalState
    local Name
    local let Online=0
    local let Offline=0
    local let POnline=0
    local let POffline=0
    local let Other=0
    local let NOnline=0
    local let NOffline=0
    local let Match=0
    local let Count=0
    local savedDirOpState=0
    local savedStandbyOpState=0
    local frmstandbyIgnored=0

    # get the NominalState and OpState for all resource groups. The OpState
    # values should all either be online or offline.

    TmpFile=/tmp/20_HAbkrt_sh.$$

    CT_MANAGEMENT_SCOPE=2 lsrsrc-api -D ' ' -s IBM.ResourceGroup::::Name::OpState::NominalState 2> /dev/null > $TmpFile
    RC=$?

    if [ $RC -eq 0 ]
    then
        while read Name OpState NominalState
        do
            if [ "$Name" = "$SMHA_DIRECTOR_RG" ]; then
                savedDirOpState=$OpState
            fi
            if [ "$Name" = "frmstandby-rg" ]; then
                savedStandbyOpState=$OpState
            fi

            Count=$(( Count + 1 ))

            case $OpState in
                1) Online=$(( Online + 1 )) ;;
                2) Offline=$(( Offline + 1 )) ;;
                5) POnline=$(( POnline + 1 )) ;;
                6) POffline=$(( POffline + 1 )) ;;
                *) Other=$(( Other + 1 )) ;;
            esac

            case $NominalState in
                0) NOnline=$(( NOnline + 1 ))
                   if [ $OpState -eq 1 ]
                   then
                       Match=$(( Match + 1 ))
                   fi ;;
                1) NOffline=$(( NOffline + 1 ))
                   if [ $OpState -eq 2 ]
                   then
                       Match=$(( Match + 1 ))
                   fi ;;
            esac

        done < $TmpFile
        rm -f $TmpFile
    else
        rm -f $TmpFile
        debug "Cannot fetch Resource Group state RC=$RC"
        return 9
    fi

    if [ $savedStandbyOpState -eq 3 ]; then
        ignore_frmstandby_rg $savedDirOpState
        frmstandbyIgnored=$?
        if [ $frmstandbyIgnored -ne 0 ]; then
            Count=$(( Count - 1 ))      # reducing the count effectively ignores the failed offline frmstandby-rg
        fi
    fi

    if (( POnline > 0 || POffline > 0 ))
    then
        debug "Resource Groups changing state: pending online = $POnline  pending offline = $POffline"
        return 2
    fi

    if (( Online == Count ))
    then
        if [ $frmstandbyIgnored -eq 2 ]; then
            debug "Resource Groups all online: online = $Online (frmstandby-rg ignored)"
            return 2
        else
            debug "Resource Groups all online: online = $Online"
            return 0
        fi
    fi

    if (( Offline == Count ))
    then
        debug "Resource Groups all offline: offline = $Offline"
        return 1
    fi

    if (( Match == Count ))
    then
        if [ $frmstandbyIgnored -eq 2 ]; then
            debug "Resource Groups all match their OpState to NominalState: online = $Online, offline = $Offline (frmstandby-rg ignored)"
            return 2
        else
            debug "Resource Groups all match their OpState to NominalState: online = $Online, offline = $Offline"
            return 3
        fi
    fi

    # any other condition is an error

    debug "Resource Groups problem: Count=$Count Online=$Online Offline=$Offline NOnline=$NOnline NOffline=$NOffline"
    return 9
}

# The following function is used to determine if the calling process is
# running on the "passive" node, with a check for errors
#
# Return value:  0   this is the active node
#                1   this is the passive node
#                2   resource not found; indicates HA config in progress
#                9   RMC error

check_passive_state () {
    local OpState
    local RC
    local Sel="Name == \"drbdp-mirror0\" && ResourceType == 0"

    OpState=$(CT_MANAGEMENT_SCOPE=1 lsrsrc-api -s IBM.Application::"${Sel}"::OpState 2> /dev/null)
    RC=$?
    if [ $RC -ne 0 ]
    then
        if [ $RC -eq 6 ]
        then
            return 2
        else
            debug "check_passive_state call to lsrsrc-api returned $RC, and stdout was $OpState, returning 9 to indicate error"
            return 9
        fi
    fi

    case $OpState in
        # 0=unknown 1=online 2=offline 3=failed_offline 4=stuck_online 5=pending_online 6=pending_offline
        1|5 ) return 0 ;;
        *   ) return 1 ;;
    esac

}

isLinkLocalAddr () {
    local addressToCheck=$1
    local firstThreeChars=${addressToCheck:0:3}
    firstThreeChars=`echo $firstThreeChars | tr A-Z a-z`
    local indexFirstColon=`expr index $addressToCheck :`
    if [ "$indexFirstColon" == "5" ]; then
        if [ "$firstThreeChars" == "fe8" -o "$firstThreeChars" == "fe9" -o "$firstThreeChars" == "fea" -o "$firstThreeChars" == "feb" ]; then
            return 1
        else
            return 0
        fi
    fi
}

expandIPv6Address()
{
    RETURN_EXPANDED_ADDR=
    local addr=$1
    local numColons=`echo $addr | tr -dc ':' | wc -c`
    local numSetsOfZeros=`expr 8 - $numColons`
    local addrLength=${#addr}
    addrLength=`expr $addrLength - 1`
    local thePart=
    for i in `seq 0 $addrLength`
    do
        local theChar=${addr:$i:1}
        if [ "$theChar" == ":" ]; then
            if [ ${#thePart} -ne 0 ]; then
                # add the part padded with zeros if necessary
                local partLen=${#thePart}
                local paddingNeeded=`expr 4 - $partLen`
                if [ "$paddingNeeded" == "3" ]; then
                    RETURN_EXPANDED_ADDR=${RETURN_EXPANDED_ADDR}000${thePart}:
                elif [ "$paddingNeeded" == "2" ]; then
                    RETURN_EXPANDED_ADDR=${RETURN_EXPANDED_ADDR}00${thePart}:
                elif [ "$paddingNeeded" == "1" ]; then
                    RETURN_EXPANDED_ADDR=${RETURN_EXPANDED_ADDR}0${thePart}:
                else
                    RETURN_EXPANDED_ADDR=${RETURN_EXPANDED_ADDR}${thePart}:
                fi

                thePart=
            elif [ "$i" != "0" ]; then
                for j in `seq 1 $numSetsOfZeros`
                do
                    RETURN_EXPANDED_ADDR=${RETURN_EXPANDED_ADDR}0000:
                done
            else
                RETURN_EXPANDED_ADDR=${RETURN_EXPANDED_ADDR}0000:
            fi
        else
            thePart=${thePart}${theChar}
        fi
    done

    if [ ${#thePart} -ne 0 ]; then
        local partLen=${#thePart}
        local paddingNeeded=`expr 4 - $partLen`
        if [ "$paddingNeeded" == "3" ]; then
            RETURN_EXPANDED_ADDR=${RETURN_EXPANDED_ADDR}000${thePart}
        elif [ "$paddingNeeded" == "2" ]; then
            RETURN_EXPANDED_ADDR=${RETURN_EXPANDED_ADDR}00${thePart}
        elif [ "$paddingNeeded" == "1" ]; then
            RETURN_EXPANDED_ADDR=${RETURN_EXPANDED_ADDR}0${thePart}
        else
            RETURN_EXPANDED_ADDR=${RETURN_EXPANDED_ADDR}${thePart}
        fi
    else
        RETURN_EXPANDED_ADDR=${RETURN_EXPANDED_ADDR}0000
    fi

    RETURN_EXPANDED_ADDR=`echo $RETURN_EXPANDED_ADDR | tr A-Z a-z`
}

# The following function saves the NominalState for all resource groups into the specified file.
#
# Return value:  0   state info has been saved
#                1   error

save_rgs_state () {
    local RC
    local SaveFile

    SaveFile=$1

    CT_MANAGEMENT_SCOPE=2 lsrsrc-api -D ' ' -s IBM.ResourceGroup::::Name::NominalState 2> /dev/null > $SaveFile
    RC=$?

    if [ $RC -ne 0 ]
    then
        debug "Cannot fetch Resource Group state RC=$RC"
        return 1
    fi

    return 0
}

# The following function restores the NominalState for all resource groups from the specified file.
#
# Return value:  0   state info has been restored
#                1   error

restore_rgs_state () {
    local SaveFile
    local RC=0

    SaveFile=$1

    if [ -s $SaveFile ]
    then
        while read Name NominalState
        do
            case $NominalState in
                0) ch_rg_online $Name    
                   RC=$?
                   ;;

                1) take_rg_offline $Name 
                   RC=$?
                   ;;

                *) RC=99         # fail safe
                   ;;
            esac

            if [ $RC -ne 0 ]
            then
                debug "Cannot set NominalState to $NominalState for $Name, RC=$RC"
                RC=1
                break
            fi

        done < $SaveFile
    else
        debug "The file $SaveFile either not not exist or is empty"
        RC=1
    fi

    return $RC
}

# The following function is used to fetch images from a Director jar file,
# which may have been updated with another product name

get_OEM_dirc_images () {
    local Image_Target_Dir=${SMHA_DIR}/html

    if [ ! -a ${Image_Target_Dir}/images/ISD_title.png ]
    then
        # extract images from Director jar file, which may have been
        # updated with another product name

        DJARFILE=$(ls -r /opt/ibm/director/lwi/runtime/isc/eclipse/plugins/com.ibm.isclite_*.jar | head -1)

        (cd $Image_Target_Dir; /opt/ibm/java-i386-60/bin/jar xf $DJARFILE images/ISD_title.png images/systemsDirector.png)
    fi

    # now generate the HTML files for the passive node, as the cons user ID

    su -c "${SMHA_SCRIPTS}/smhapassivehtmlgenerator.sh HA" cons

    return 0
}

# The following function is used to find all the resource groups which contain resources that have a 
# DependsOn relationship to the specified target resource. This function is "recursive", in that all
# DependsOn relationships, going up the dependency tree, are examined.
#
# N.B it is assumed that the specified resource is a member of IBM.Application and that it is a non-fixed
# resource
#
# Input:  Arg1 - target resource name
#
# Exit code:   0 - Dependent resource group names written to STDOUT
#              1 - error
#              2 - no resource groups found

find_Dependent_RGs () {
    local TargetFile=/tmp/smha_fdp1.$$
    local SourceFile=/tmp/smha_fdp2.$$
    local RGFile=/tmp/smha_fdp3.$$
    local RC=2                        # assume no dependent RGs found

    rm -f $TargetFile $SourceFile $RGFile

    RsrcName=$1           # get initial target resource name

    if ! lsrsrc-api -s "IBM.Application::Name == '${RsrcName}' && ResourceType != 0::" > $TargetFile 2> /dev/null
    then
        debug "$RsrcName not found"
        rm -f $TargetFile
        return 1
    fi

    debug "TargetRH=$(cat $TargetFile)"

    while [ -f $TargetFile ]
    do
        while read TargetRH
        do
            # find all Source RHs for this Target RH

            lsrsrc-api -D ' ' -s "IBM.ManagedRelationship::Target |< {'$TargetRH'} && Relationship == 7::Name::Source" 2> /dev/null | \
            while read RelName SourceRH
            do
                debug "RelName=$RelName  SourceRH=$SourceRH"

                # get ResourceGroup that contains this SourceRH

                if ! lsrsrc-api -s "IBM.ManagedResource::ManagedResource == '$SourceRH'::MemberOf" >> $RGFile 2> /dev/null
                then
                    debug "RG name not found for managed resource $SourceRH"
                    exit 1         # must use exit since this while loop is in a sub-shell
                fi

                echo "$SourceRH" >> $SourceFile
            done

            if [ $? -ne 0 ]
            then
                debug "Could not find Source or RG of Source"
                rm -f $TargetFile $SourceFile $RGFile
                return 1
            fi
        done < $TargetFile

        if [ -f $SourceFile ]
        then
            # remove duplicate Source RHs and make then a new set of Target RHs to examine
            LC_ALL=C sort -u -o $TargetFile $SourceFile
            rm -f $SourceFile
        else
            rm -f $TargetFile        # all done
        fi
    done

    if [ -s $RGFile ]
    then
        debug "ResourceGroups found:" "$(cat $RGFile)"
        LC_ALL=C sort -u $RGFile             # remove duplicates and write to STDOUT
        RC=0
    fi
    rm -f $RGFile

    return $RC
}

# I split this into mirrored and non-mirrored so I can deal with 
# upgrade considerations (can't touch mirrored on passive node)
deactivate_dir_server_nonstop () {
    deactivate_dir_server_nonstop_mirrored_files
    deactivate_dir_server_nonstop_non_mirrored_files
}

deactivate_dir_server_nonstop_mirrored_files () {
   local iscRuntimePropFile=$INSTALL_DIR/lwi/conf/overrides/ISCRuntime.properties
   #remove the nonstop profile entry from ISCRuntime.properties
   perl -pi -e "s/,nonstop//g" $iscRuntimePropFile
   perl -pi -e "s/nonstop,//g" $iscRuntimePropFile
}

deactivate_dir_server_nonstop_non_mirrored_files () {
   local nonstopFlagFile=$INSTALL_DIR/proddata/nonstop
   #remove the nonstop flag file
   rm -rf $nonstopFlagFile
}

activate_dir_server_nonstop () {
    local iscRuntimePropFile=$INSTALL_DIR/lwi/conf/overrides/ISCRuntime.properties
    local nonstopFlagFile=$INSTALL_DIR/proddata/nonstop

    #add the nonstop profile entry to ISCRuntime.properties if it isn't there
    if [ -f $iscRuntimePropFile ]; then
        local option_line=`grep "profile.isc*=" $iscRuntimePropFile`
        if [ -n "$option_line" ]; then
            echo $option_line | grep nonstop 1>/dev/null 2>/dev/null
            rcEcho=$?
            # if nonstop profile is not in the list, then add it
            if [ $rcEcho -eq 1 ]; then
                # Get the value: delete everything up to the first = from the left
                oldProp=${option_line#*=}
                newProp=${oldProp},nonstop
                perl -pi -e "s/$oldProp/$newProp/g" $iscRuntimePropFile
            fi
        fi
    fi


    #add the nonstop flag file
    touch $nonstopFlagFile
    chown root:root $nonstopFlagFile
    chmod 644 $nonstopFlagFile
}

# The following function is used to create resources of the IBM.Condition,
# IBM.EventResponse and IBM.Association resource classes. One resource of each
# class is created on the specified node. A Condition resource is the specification
# of an event of interest. An EventResponse resource specifies a script to be run
# when an event occurs. An Association, also known as a Condition Response, links a
# Condition to an EventResponse. Such associations may be 1 to N, N to 1 or N to M,
# but this function creates 1 to 1 associations. These resources and their associated
# commands are thoroughly documented in the RSCT publications.
#
# Input: Arg 1 - class of resource(s) to be monitored
#        Arg 2 - event expression
#        Arg 3 - description of condition
#        Arg 4 - select string for resources to be monitored
#        Arg 5 - node where resource is to be created
#        Arg 6 - ID used to create names
#        Arg 7 - Script to be run (command invocation)
#
# Exit code: 0 - success
#            1 - error. Output of RSCT command is logged

mk_condresp () {
    local Class=$1
    local Expression="$2"
    local Description="$3"
    local SelectString="$4"
    local TargetNode=$5
    local ID=$6
    local Script="$7"
    local cmdout=
    local cmdrc=0

    debug "Make ConditionResponse: $Class \"${Description}\" \"${Expression}\" \"${SelectString}\" $TargetNode $ID"

    cmdout=$(mkcondition -r $Class \
                     -e "$Expression" \
                     -d "$Description" \
                     -m p \
                     -g 2 \
                     -s "$SelectString" \
                     -p $TargetNode \
                     SMHAmonitor_${ID} 2>&1)
    cmdrc=$?
    if [ $cmdrc -ne 0 ]; then
        debug "mkcondition error $cmdrc: $cmdout"
        return 1
    fi

    cmdout=$(mkresponse -n process_${ID} \
                   -e A \
                   -p $TargetNode \
                   -s "$Script" \
                   SMHAprocess_${ID} 2>&1)
    cmdrc=$?
    if [ $cmdrc -ne 0 ]; then
        debug "mkresponse error $cmdrc: $cmdout"
        return 1
    fi

    cmdout=$(mkcondresp SMHAmonitor_${ID}:${TargetNode} SMHAprocess_${ID} 2>&1)
    cmdrc=$?
    if [ $cmdrc -ne 0 ]; then
        debug "mkcondresp error $cmdrc: $cmdout"
        return 1
    fi

    return 0
}

rm_condresp () {
    local TargetNode=$1
    local ID=$2
    local cmdout=
    local cmdrc=0

    debug "Remove ConditionResponse: $TargetNode $ID"

    cmdout=$(rmcondresp SMHAmonitor_${ID}:${TargetNode} 2>&1)
    cmdrc=$?
    if [ $cmdrc -ne 0 ]; then
        debug "rmcondresp error $cmdrc: $cmdout"
    fi

    cmdout=$(rmresponse SMHAprocess_${ID}:${TargetNode} 2>&1)
    cmdrc=$?
    if [ $cmdrc -ne 0 ]; then
        debug "rmresponse error $cmdrc: $cmdout"
    fi

    cmdout=$(rmcondition SMHAmonitor_${ID}:${TargetNode} 2>&1)
    cmdrc=$?
    if [ $cmdrc -ne 0 ]; then
        debug "rmcondition error $cmdrc: $cmdout"
    fi

    return 0
}

start_condresp () {
    local TargetNode=$1
    local ID=$2
    local cmdout=
    local cmdrc=0

    debug "Start ConditionResponse: $TargetNode $ID"

    cmdout=$(startcondresp SMHAmonitor_${ID}:${TargetNode} SMHAprocess_${ID} 2>&1)
    cmdrc=$?
    if [ $cmdrc -ne 0 ]; then
        debug "startcondresp error $cmdrc: $cmdout"
        return 1
    fi

    return 0
}

stop_condresp () {
    local TargetNode=$1
    local ID=$2
    local cmdout=
    local cmdrc=0

    debug "Stop ConditionResponse: $TargetNode $ID"

    cmdout=$(stopcondresp SMHAmonitor_${ID}:${TargetNode} SMHAprocess_${ID} 2>&1)
    cmdrc=$?
    if [ $cmdrc -ne 0 ]; then
        debug "stopcondresp error $cmdrc: $cmdout"
    fi

    return 0
}

# Create conditions / responses to monitor the drbdm-mirror0 resource, such that
# "plug ins" can be invoked when the local constituent resource becomes online or
# pending offline
#
# Input: Arg1 - if 0, create resources on both nodes
#               if 1, only create resources on the remote node (replace a node)

setup_monitoring_for_mirror0 () {
    local rc=0
    local Expr="OpState != OpState@P && (OpState == $TSA_ONLINE || OpState == $TSA_OFFLINE)"
    local Desc="Monitor ${SMHA_DRBDM_MIRROR0_RSRC_NAME}"
    local SelStrPfx="Name == '${SMHA_DRBDM_MIRROR0_RSRC_NAME}' && ResourceType == 0"

    if [ $1 -eq 0 ]; then
        # create the local resources
    
        get_local_rsct_node_name
    
        # in case this is a retry, stop/remove the old resources first
    
        stop_condresp $LOCAL_RSCT_NODE_NAME $SMHA_DRBDM_MIRROR0_RSRC_NAME 
        rm_condresp $LOCAL_RSCT_NODE_NAME $SMHA_DRBDM_MIRROR0_RSRC_NAME 
    
        mk_condresp IBM.Application \
                    "$Expr" \
                    "$Desc" \
                    "${SelStrPfx} && NodeNameList |< {'$LOCAL_RSCT_NODE_NAME'}" \
                    $LOCAL_RSCT_NODE_NAME \
                    $SMHA_DRBDM_MIRROR0_RSRC_NAME \
                    /opt/ibm/director/ha/bin/priv/smhaDRBDMevent.sh
        rc=$?
        if [ $rc -ne 0 ]; then
            debug "Cannot create mirrored storage monitor on the local node."
            return 1
        fi
    
        start_condresp $LOCAL_RSCT_NODE_NAME $SMHA_DRBDM_MIRROR0_RSRC_NAME 
        rc=$?
        if [ $rc -ne 0 ]; then
            debug "Cannot start mirrored storage monitor on the local node."
            return 1
        fi
    fi

    # create the remote resources

    get_remote_rsct_node_name

    # in case this is a retry, remove the old resources first

    stop_condresp $REMOTE_RSCT_NODE_NAME $SMHA_DRBDM_MIRROR0_RSRC_NAME 
    rm_condresp $REMOTE_RSCT_NODE_NAME $SMHA_DRBDM_MIRROR0_RSRC_NAME 

    mk_condresp IBM.Application \
                "$Expr" \
                "$Desc" \
                "${SelStrPfx} && NodeNameList |< {'$REMOTE_RSCT_NODE_NAME'}" \
                $REMOTE_RSCT_NODE_NAME \
                $SMHA_DRBDM_MIRROR0_RSRC_NAME \
                /opt/ibm/director/ha/bin/priv/smhaDRBDMevent.sh
    rc=$?
    if [ $rc -ne 0 ]; then
        debug "Cannot create mirrored storage monitor on the remote node."
        return 1
    fi

    start_condresp $REMOTE_RSCT_NODE_NAME $SMHA_DRBDM_MIRROR0_RSRC_NAME 
    rc=$?
    if [ $rc -ne 0 ]; then
        debug "Cannot start mirrored storage monitor on the remote node."
        return 1
    fi

    return 0
}

# Remove conditions / responses to monitor the drbdm-mirror0 resource.

remove_monitoring_for_mirror0 () {
    local rc=0

    # remove the local resources

    get_local_rsct_node_name

    stop_condresp $LOCAL_RSCT_NODE_NAME $SMHA_DRBDM_MIRROR0_RSRC_NAME 
    rm_condresp $LOCAL_RSCT_NODE_NAME $SMHA_DRBDM_MIRROR0_RSRC_NAME 

    # remove the remote resources

    get_remote_rsct_node_name

    # in case this is a retry, remove the old resources first

    stop_condresp $REMOTE_RSCT_NODE_NAME $SMHA_DRBDM_MIRROR0_RSRC_NAME 
    rm_condresp $REMOTE_RSCT_NODE_NAME $SMHA_DRBDM_MIRROR0_RSRC_NAME 
}

# the following function is used to see if one or more files that match a file path prefix
# exist.
#
# Input: path prefix
#
# Returns: 0 if one or more files match the prefix
#          1 if none match

files_exist () {
    local filepfx=$1
    local lsout
    local rc=0

    lsout=$(ls ${filepfx}* 2> /dev/null)
    debug "Files exist: $lsout"

    if [ -z "$lsout" ]; then
        rc=1
    fi

    return $rc
}

get_dir_pid () {
	# Don't count on the return code value!!! Use global DIRECTOR_PID variable!
    
    DIRECTOR_PID=`ps auxww | grep LaunchLWI |  grep -v grep | grep -v profile.standby | grep -v -i "/agent/" | grep -i "director/jre/bin"  | awk '{print $2}'`
    debug "get_dir_pid: Director pid is: $DIRECTOR_PID"
    if [ -n "$DIRECTOR_PID" ]; then
        return 1
    else
        debug "get_dir_pid: return 0"
        return 0
    fi
}

get_dir_server_nonstop_pid ()
{
	# Don't count on the return code value!!! Use global NONSTOP_PID variable!
    
    NONSTOP_PID=`ps auxww | grep nonstop_linux |  grep -v grep | grep -v agent | awk '{print $2}'`
    debug "get_dir_server_nonstop_pid: nonstop process pid is: $NONSTOP_PID"
    if [ -n "$NONSTOP_PID" ]; then
        return 1
    else
        debug "get_dir_server_nonstop_pid: return 0"
        return 0
    fi
}

# Stop the director server.  This function is only meant to be used when
# the system is NOT in a HA environment.  It has not been tested for use
# in a HA environment, and might not work properly
stop_director_server ()
{
    debug "Entering stop_director_server"
	SMSTOP_CMD=$INSTALL_DIR/bin/smstop
	
    for (( i = 0; i < 4; i++ ))
    do
        smstop_out=`$SMSTOP_CMD`
        smstop_rc=$?
        debug "smstop rc=$smstop_rc, stdout=$smstop_out"
        sleep 10
        
        #get DIRECTOR_PID if the process is running
        get_dir_pid
        
        #get NONSTOP_PID if the process is running
        get_dir_server_nonstop_pid
        
        # if both dir server and nonstop processes are gone, return success
        if [ -z "$DIRECTOR_PID" -a -z "$NONSTOP_PID" ]; then
            return 1;
        fi
        sleep 30
    done

    for (( i = 0; i < 4; i++ ))
    do
        debug "smstop was not able to stop director and/or nonstop process, so trying to kill the processes: dir=$DIRECTOR_PID, nonstop=$NONSTOP_PID"

        signal_flag=
        if [ $i -gt 1 ]; then
            signal_flag="-9"
        fi
        kill_out=`kill $signal_flag $DIRECTOR_PID $NONSTOP_PID`
        kill_rc=$?
        debug "kill rc=$kill_rc, stdout=$kill_out"
        
        sleep 20
        
        #get DIRECTOR_PID if the process is running
        get_dir_pid
        
        #get NONSTOP_PID if the process is running
        get_dir_server_nonstop_pid
        
        # if both dir server and nonstop processes are gone, return success
        if [ -z "$DIRECTOR_PID" -a -z "$NONSTOP_PID" ]; then
            return 1;
        fi
        sleep 20
    done

	debug "Unable to kill both the director server process and the nonstop server process, so returning 0"
	return 0
}

# The following function check to see if the peer node has been excluded from the domain, i.e. it is
# listed in the ExcludeNodes class attribute of the IBM.CHARMControl class. If so, the peer node name
# is written to stdout
#
# N.B. this function assume the attribute, which is an array, is either null or only contains one  value
# 
# Return value:  0 - success, stdout is either the peer node name or NULL
#                1 - could not determine if the peer node is excluded or ExcludeNodes contains
#                    an unexpected value
# 

get_excluded_node_name () {
    local ret_code=1
    local RC
    local exnodelist
    local RH
    local rNodeName

    exnodelist=$(lsrsrc-api -c IBM.CHARMControl::ExcludedNodes 2> /dev/null)
    RC=$?

    if [ $RC -eq 0 ]; then
        debug "ExcludedNodes list is $exnodelist"
        RH=${exnodelist#*\{}
        RH=${RH%\}*}
        if [ -n "$RH" ]; then
            rNodeName=$(lsrsrc-api -r "$RH"::Name 2> /dev/null)
            RC=$?
            if [ $RC -eq 0 ]; then
                debug "Remote node name is $rNodeName"
                get_remote_rsct_node_name
                if [ "$REMOTE_RSCT_NODE_NAME" = "$rNodeName" ]; then
                    echo "$rNodeName"
                    ret_code=0
                fi
            else
                debug "Cannot fetch remote node name, RC=$RC"
            fi
        else
            ret_code=0      # the ExcludedNodes list is empty
        fi
    else
        debug "Cannot fetch ExcludedNodes, RC=$RC"
    fi

    return $ret_code
}

# The following function obtains the RSCT normalized (canonical) form of the IP
# address specified by the single function argument
#
# The global shell symbol NORMALIZED_IP_ADDRESS is set by this function
# if the input IP address is valid
#
# Return value:   0 - NORMALIZED_IP_ADDRESS is set to canonical form of input IP address
#                 1 - input IP address is not valid
#
# N.B. the ct_normalize_ipaddr shipped in the priv directory will eventually be replaced
# by the same named command shipped with RSCT.

NORMALIZE_IP_CMD=${SMHA_SCRIPTS}/priv/ct_normalize_ipaddr

get_canonical_IP () {
    local IPaddress=$1
    local rc

    NORMALIZED_IP_ADDRESS=$($NORMALIZE_IP_CMD $IPaddress 2> /dev/null)
    rc=$?
    if [ $rc -ne 0 ]; then
        debug "$NORMALIZE_IP_CMD failed for $IPaddress, rc=$rc"
        return 1
    fi
    return 0
}
