#!/bin/bash
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2011, 2011 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
# IBM_COPYRIGHT_END

#
# 09N6996
#

# this script contains actions to be taken as part of the post
# install processing of isdvmi-ha package. It is invoked by
# the scriplet therein.

# Ensure that the RMC well-known port 657 is in the blacklist
# file that prevents the port being returned by the bindrevport()
# function. Currently, this blacklist file is only supported on
# newer releases of SUSE Linux (SLES)

INSTALL_ARG=$1         # arg passed to RPM scriplet

BLACKLIST=/etc/bindresvport.blacklist
TMPBLACKLIST=${BLACKLIST}.tmp.$$
TMPCRONTAB=/tmp/crontab.$$
APACHE2CFG=/opt/ibm/vmi/data/disklayout.d/apache2.cfg
MirrorPath=/mirror0/os/apache2
FileSysPath=/etc/apache2
MirrorSysCfgPath=/mirror0/os/sysconfig
MirrorSysCfgFile=/mirror0/os/sysconfig/apache2
SysCfgFile=/etc/sysconfig/apache2

TABCHAR="	"                   # a single tab character
BLANKCHAR=" "                   # a single blank character

if [[ -f $BLACKLIST ]]
then
    cp -p $BLACKLIST $TMPBLACKLIST      # copy to temp file

    if ! grep "^[${BLANKCHAR}${TABCHAR}]*657" $TMPBLACKLIST >& /dev/null
    then
        echo "657${TABCHAR}# rmc" >> $TMPBLACKLIST
        mv $TMPBLACKLIST $BLACKLIST
    else
        rm -f $TMPBLACKLIST
    fi
fi

# add a crontab entry for root to trim the trace spool areas
# every 15 minutes

if /usr/bin/crontab -u root -l > $TMPCRONTAB 2> /dev/null
then
    grep -q 'smhachkspool.sh' $TMPCRONTAB 2> /dev/null
    if [ $? -ne 0 ]
    then
        echo "0,15,30,45 * * * * /opt/ibm/director/ha/bin/install/smhachkspool.sh" >> $TMPCRONTAB
        /usr/bin/crontab -u root $TMPCRONTAB
    fi
fi
rm -f $TMPCRONTAB

# Set up the bind mount for the Apache server
# N.B. Eventually, this bind mount must be made part of CSDA, at the next CSDA release
# that is a full replacement image, not an update.

if [ ! -f $APACHE2CFG ]
then
    echo "bind:${MirrorPath}:${FileSysPath}" > $APACHE2CFG
    chmod 644 $APACHE2CFG
    chown root:root $APACHE2CFG

    # update /etc/fstab with the new mount point.

    if [ -e /opt/ibm/vmi/bin/updatefstab2 ]; then
        /opt/ibm/vmi/bin/updatefstab2 -w
    else
        /opt/ibm/vmi/bin/updatefstab -w
    fi

    # check if HA is configured; if so, is this the Active node ??

    /usr/smrshbin/smhastatus >& /dev/null
    RC=$?
    if [ $RC -eq 0 -o $RC -eq 4 ]; then
        # this is either the Active node or HA is not configured

        # make the directory in the mirror, and then copy the data thereto

        if [ ! -d $MirrorPath ]; then
            mkdir -p $MirrorPath
            cp -fr -p ${FileSysPath}/* $MirrorPath
        fi

        mount $FileSysPath

        # The Apache server has another config file that must be copied into the mirror, then replaced
        # by a symlink to the copy.

        if [ ! -d $MirrorSysCfgPath ]; then
            mkdir -p $MirrorSysCfgPath
            mv $SysCfgFile $MirrorSysCfgFile
        fi
    fi

    # Make the symlink on both the active and passive nodes or if HA not configured

    ln -sf $MirrorSysCfgFile $SysCfgFile
fi

if [[ -d /tmp/kgnmas ]]
then
    echo "INSTALL_ARG=$INSTALL_ARG" > /tmp/kgnmas/smhaposti.out
fi

exit 0
