#!/bin/bash
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2009, 2012 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
# IBM_COPYRIGHT_END 
 
PATH=/usr/bin:$PATH
PROGNAME=$(basename $0)
PROGPATH=$(dirname $0)

#### Standard code to find an pull in all sm common code ####
INSTALL_DIR=/opt/ibm/director
CMD_DIR=$INSTALL_DIR/ha/bin/
COMMON_CODE=/opt/ibm/director/ha/bin/include/smhacommon.sh

if [ -f $COMMON_CODE ]; then
    source $COMMON_CODE
else
    echo "$PROGNAME:Common code not found."
    exit 1
fi
#############################################################
load_common_code $INSTALL_DIR/ha/bin/include/smhaparms.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhaerrcode.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhainitcom.sh

PARM_FLAGS="B prop2rollback F prop2failed"
PARM_MULTI=""
PARM_TRANSLATION="R resource \
U user \
V prop1 \
W prop2 \
B prop2rollback \
F prop2failed"
#PARM_SINGLES="R resource"
PARM_VALID=$PARM_TRANSLATION
INIT_SCRIPT=$SMHA_SCRIPTS/priv/smhainit.sh
###################################
## Functions                     ##
###################################
proc_parms () {
	
	# Parse the parameters
	# The result should be that variables are set for each parameter and
	# CMD_PARMS will contain the list of parameters processed
    parse_all_parms $*
    
}

#####################################################
# Process the resource parameter
#####################################################
proc_resource () {
    
    if [ -n "$resource" ]; then
        # Run through each resource that is supported
        case "$resource" in
            firewall|FIREWALL )
                $INIT_SCRIPT --firewall
                RC=$?                
            ;;
            users|USERS )
                $INIT_SCRIPT --users
                RC=$?
            ;;
            dns|DNS )
                $INIT_SCRIPT --dns
                RC=$?
            ;;
            ntp|NTP )
                $INIT_SCRIPT --ntp
                RC=$?
            ;;
            authmech|AUTHMECH )
                $INIT_SCRIPT --authentication-mech
                RC=$?
            ;;
             groups|GROUPS )
                $INIT_SCRIPT --groups
                RC=$?
            ;;
            fwinterfaces|FWINTERFACES )
                $INIT_SCRIPT --update-interfaces
                RC=$?
            ;;
            syslog|SYSLOG )
                $INIT_SCRIPT --syslog
                RC=$?
            ;;
            ldapuser|LDAPUSER )
                ensure_parms "user"
                reconcileUser $user
                RC=$?
            ;;
            db2user|DB2USER )
                ensure_parms "user"
                reconcileUser $user
                RC=$?
            ;;
            mirror-required-passive-dirs|MIRROR-REQUIRED-PASSIVE-DIRS )
                $INIT_SCRIPT --mirror-required-passive-dirs
                RC=$?
            ;;
            sync-agent-manager|SYNC-AGENT-MANAGER )
                local properties=
                if [ -n "$prop1" ]; then
                    properties="--prop1 ${prop1}"
                fi
                if [ -n "$prop2" ]; then
                    properties="${properties} --prop2 ${prop2}"
                fi
                if [ -n "$prop2rollback" ]; then
                    properties="${properties} --prop2rollback"
                fi
                if [ -n "$prop2failed" ]; then
                    properties="${properties} --prop2failed"
                fi
                
                is_configured
                if [ $? -eq 1 ]; then
                    run_init "sync-agent-manager" $properties
                    RC=$?
                elif [ -n "$properties" ]; then
                    # Just store the properties
                    run_init "store-props" $properties
                    RC=$?
                fi
            ;;
            
            *)
                debug "Sync request of unknown service: $resource"
                RC=$BAD_PARAMETER_VALUE
            ;;
        esac
 
        debug "smhasync.sh: exiting with RC: $RC"
        exit $RC
    fi
    
}

reconcileUser () {
	debug "reconcileUser called with user $1"
	# I should have used "reconcileuser" when I did this first for
	# LDAP users.  Oh well, LDAP or DB2, both run through 
	# "reconcileUser" on the peer node.
	run_init "ldapuser" ${1} 
	RC=$?
	debug "reconcileUser for user $1 returned: $RC"
	return $RC
}

###################################
## Main code                     ##
###################################
proc_parms $*

# Check if HA is configured, if not exit
is_configured
if [ $? -eq 1 ]; then 
    # Process the resource parameter
    proc_resource
elif [ "$resource" == "sync-agent-manager" -o "$resource" == "SYNC-AGENT-MANAGER" ]; then
    # We want to store the props even if HA is not currently configured
    if [ -n "$prop1" -o -n "$prop2" -o -n "$prop2failed" -o -n "$prop2rollback" ]; then
        proc_resource
    fi
else
    exit $HA_ENVIRONMENT_NOT_CONFIGURED
fi

exit 0


