#!/usr/bin/perl -w
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2009, 2011 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.


use strict;
package lib::Roles;

 
my $GROUP_FILE="/etc/group";

my %roles=(
    smmon        => "smmon",
    smmgr        => "smmgr",
    smadmin      => "smadmin",
    smuser       => "smuser",
    groupread    => "groupread",
    smservicerep => "smservicerep"
    );

#Return the Group ID from the Group name
sub getRoleId{
	open(HANDLE, $GROUP_FILE);
	my ($role_name) = @_;
	
	while(<HANDLE>) {
		if (/^${role_name}:/) {
			print $_;
			close HANDLE;
			my @fields = split(":", $_);
			return $fields[2];
		}
	}
}

#Return the Group name from the Group ID
sub getRoleName {
	open(HANDLE, $GROUP_FILE);
	my ($group_id) = @_;
	
	while(<HANDLE>) {
		print $_;
		if (/:${group_id}:/) {
			print $_;
			close HANDLE;
			my @fields = split(":", $_);
			return $fields[0];
		}
	}
	close HANDLE;
}

#if the director server is up, we compare against Director's known roles
#if the director server is down, we compare against a default set of roles 
sub in_defined_roles {
	my @roles_array;
	my ($role_names) = @_;
	my @directorRoles;
	if ($role_names =~ /,/) {
		@roles_array = split(',',$role_names);
	}
	if (@roles_array) {
	   foreach (@roles_array) {
		  if(!exists($roles{$_})) {
			 return 0;
		  }
	   }
	}
	else {
		if(!exists($roles{$role_names})) {
            return 0;
	 	}
	}
	return 1;	
}

sub in_director_roles {
	my @roles_array;
	my %director_roles_hash;
	my ($role_names, $given_roles) = @_;
	
	if ($role_names =~ /,/) {
		@roles_array = split(',',$role_names);
	}

	%director_roles_hash = map {$_ => "$_"} split(',',$given_roles);
	
	if (@roles_array) {
	   foreach (@roles_array) {
		  if(!exists($director_roles_hash{$_})) {
			 return 0;
		  }
	   }
	}
	else {
		if(!exists($director_roles_hash{$role_names})) {
            return 0;
	 	}
	}
	return 1;	
	
}

sub getUserRoles {
	my ($user) = @_;
	my @roles;
	open(HANDLE, $GROUP_FILE);
    
    while(<HANDLE>) {
        if (/${user}/) {
            my @fields = split(":", $_);
            push(@roles, $fields[0]);
        }
    }
    close HANDLE;
    
    return @roles;
}

sub canRemoveRoles {
	my ($user, $roles) = @_;
	my @roles_array;
		
	if ($roles =~ /,/) {
		@roles_array = split(',',$roles);
	}
	else {
		push(@roles_array, $roles);
	}
	
	my @assignedRoles = getUserRoles($user);
	
	if (@assignedRoles) {
		foreach (@roles_array) {
			my $role =$_;
			my @match = grep(/$role/, @assignedRoles);
			if (!@match) {
				return 0; #trying to remove a group the user does not belong to
			}
		}
		return 1;
	}
	else {  #user does not have any supplementary groups
		return 0;
	}
}


sub getDefinedRoles {
	return %roles;
}

sub extractPrimaryRole {
	my ($roles_string) = @_;
	my @roles_array;
	
	if ($roles_string =~ /,/) {
        @roles_array = split(',',$roles_string);
	}
	my @rt;
	if (@roles_array) {
		$rt[0] = $roles_array[0];
		$rt[1] = $roles_string;
		$rt[1] =~ s/$roles_array[0],//g;
	} else {
		$rt[0] = $roles_string;
        $rt[1] = '';
	}
	
	return @rt;
	
}
#Given a username, determine whether it has been assigned a specific role (passed as an argument)
sub isAssignedRole {
	my ($user, $role_to_match) = @_;
	
	my @current_roles = getUserRoles($user);
	
	foreach (@current_roles) {
		if ($_ eq $role_to_match) {
			return 1;
		}
	}
	return 0;
}
1;