#!/bin/bash
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2009, 2012 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
# IBM_COPYRIGHT_END


PROGNAME=$(basename $0)
PROGPATH=$(dirname $0)

export PATH=/opt/ibm/director/bin:/opt/ibm/director/lwi/bin:/bin:/usr/bin:/usr/sbin:/sbin

INSTALL_DIR=/opt/ibm/director
CMD_DIR=$INSTALL_DIR/ha/bin/priv
COMMON_CODE=/opt/ibm/director/ha/bin/include/smhacommon.sh

if [ -f $COMMON_CODE ]; then
    source $COMMON_CODE
else
    echo "$PROGNAME:Common code not found."
    exit 1
fi

load_common_code $INSTALL_DIR/ha/bin/include/smhaagent.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhainitcom.sh

#################################################
#  ensure_secondary_firewall_open
#   This bootstrap routine will ensure that the JMS_HA messaging
#   ports that we use are available on the secondary.
#   All other needed firewall ports will be opened when 
#   the firewall itself is duplicated.
#################################################
ensure_secondary_firewall_open () {

	SMHA_CLASS=com.ibm.vmi.ha.network.ScanNetworkPreReqs
	SM_JAVA=/opt/ibm/director/jre/bin/java
	SM_CLASSPATH=$SM_CLASSPATH:/opt/ibm/director/ha/lib/classes
	SM_CLASSPATH=$SM_CLASSPATH:/opt/ibm/director/ha/lib/smhadm.jar

    cmdAPIPath=`find /opt/ibm/director/lwi/runtime/ApplianceMgmt/eclipse/plugins/com\.ibm\.vmi\.cmdapi\.util* -name CmdAPI.jar | sort -r | head -n 1 -`
    SM_CLASSPATH=$SM_CLASSPATH:${cmdAPIPath}
	
	SM_LOGGING=-Djava.util.logging.config.file=/opt/ibm/director/ha/logs/daemon_logging.properties
	#DEBUG_ARGS="-Dsmhawait=60000 -Xdebug -Xnoagent -Xrunjdwp:transport=dt_socket,server=y,address=8000,suspend=n"
	#VERBOSE="-verbose:class"
	
	# Start the JVM
    export CLASSPATH=$SM_CLASSPATH
    $SM_JAVA $VERBOSE $DEBUG_ARGS $SM_LOGGING $SMHA_CLASS $*

    return $?
    
}

check_coreagent_file () {
    local srchd=/opt/ibm/director/agent/runtime/agent/subagents/
    local datfile=coreagent.dat

    debug "check for $datfile"
	cfile=`find $srchd -name "$datfile"`
	
	if [ -n "$cfile" ]; then	    
        debug "Found $datfile: $cfile"        	    
	else
	    debug "$datfile file not found. find result: $cfile"
	    exit 1
	fi
}

#################################################
#  Main code
#################################################
OPTERR=0
while getopts cph:a:tsrv:gO:A:m: flag;do
    case $flag in
        c) checkCAS=1 ;;
        p) pingNode=1 ;;
        h) OTHER_HOSTNAME=$OPTARG ;;
        a) OTHER_IPADDR=$OPTARG ;;       
        t) forceTimeSync=1 ;;   
        s) callSNTP=1 ;;   
        r) removeAllNTPServers=1 ;;   
        v) checkVersion=$OPTARG ;;
        g) getVersion=1 ;;
        O) operation=$OPTARG;;
        A) activations=$OPTARG;;
        m) message=$OPTARG;;

    esac
done

# Route the operations 
if [ -n "$operation" ]; then
        case $operation in
        initHA )
            $SMHA_SCRIPTS/priv/smhaprinit.sh $@
            exit $?
        ;;
        dhcp )
            $SMHA_SCRIPTS/priv/smhadhcpinit.sh $@
            exit $?
        ;;
        addrChOff )
            turn_off_address_change_processing
            exit $?
        ;;
        addrChOn )
            turn_on_address_change_processing
            exit $?
        ;;
        forceAddrChOff )
            remove_force_address_change_processing_property
            exit $?
        ;;
        updateNodesStatus )
            # Check the string constant to determine what message to show.  Note that I couldn't pass the actual
            # message as a parameter, because the messages needs to show up in this script in so that the message can make
            # it into the .po file so it can be translated.  See record_cfg_status for more details.
            if [[ "$message" == "HA_CONFIG_ERROR_MUST_RELOAD" ]]; then
                sm_msg "`eval_gettext "The high availability configuration failed. Restore both nodes from backup and try again."`"
                record_cfg_status_fatal_error "The high availability configuration failed. Restore both nodes from backup and try again."
                exit $?
            else
                if [[ "$message" == "RESET_STATUS_FILES" ]]; then
                    debug "Removing .cfgstatus file and .errorDuringConfig file the local node (if the files exist)"                
                    rm -rf $CFG_STATUS_FILE
                    rm -rf $ERROR_DURING_CONFIG_FLAGFILE
                    rm -rf $DM_CONF
                    rm -rf $BROKER_KS
                    exit 0
                else
                    debug "Could not resolve key $message to a message to put in .cfgstatus"
                    exit 2
                fi
            fi
        ;;
        getTimezone )
            # get the system's timezone
            timezoneOutput=`/opt/ibm/director/vmi/scripts/execWithPrivilege/dateTimeUtil -c queryTimezone`
            if [ $? -eq 50 ]; then
                #NOTE: output of the command should be "/usr/share/zoneinfo/<TimeZone>:"
                timezoneOutput=${timezoneOutput:21} # skip the first 21 characters, "/usr/share/timezone/
                timezoneOutput=${timezoneOutput%:*} # remove the :* from the end of the string 
                echo $timezoneOutput
                exit 0
            else
                timezoneOutput=`cat /etc/sysconfig/clock | grep TIMEZONE`
                timezoneOutput=${timezoneOutput#*\"} # remove the TIMEZONE=" from the beginning of the string 
                timezoneOutput=${timezoneOutput%\"} # remove the " from the end of the string 
                echo $timezoneOutput
                exit 0
            fi
            
            exit 1
        ;;
        revertCasIpProp )
            revert_cas_ip_property restartAgent
            exit $?
        ;;
        *)
            exit 99
        ;;
        esac
	
fi

if [ -n "$checkCAS" ]; then
    check_coreagent_file
fi

if [ "$pingNode" == "1" ]; then

    ping_host $OTHER_HOSTNAME
    RC=$?
	if [ $RC -eq 0 ]; then	
		# since we can ping the shortname, there is no reason to add anything to	
		# the hosts file.
		# Check if the JMS ports we need are open, and if they are not, attempt
		# to open them.  Returns non-zero on failure.
		ensure_secondary_firewall_open "ensureSecondaryOpen" $OTHER_IPADDR
		RC=$?
		if [ $RC -ne 0 ]; then	
			# We were not able to open the ports we need to bootstrap an
			# HA configuration on the secondary node.	
			return_error $RC "Error opening JMS_HA firewall port on secondary node, RC=$RC"	
		fi
		# All done, ports open and can ping.
		exit 0	
	fi
	
	
	ping_host $OTHER_IPADDR
	RC=$?
	if [ $RC -ne 0 ]; then	
		# Since we can not ping the IP address, there is no reason to add it to	
		# the hosts file, but we should return an error	
		# 
		return_error $RC "Error pinging primary node addr $OTHER_IPADDR from secondary node, RC=$RC"	
	fi
	
	add_etc_hosts_entry $OTHER_IPADDR $OTHER_HOSTNAME
		
  	# If we are here, then we could not ping using the shortname, but we
  	# could ping using the IP address.  We've added the name to the 
  	# hosts file, and now we need to ensure that the JMS ports that 
  	# we need on the secondary node are open.
  	ensure_secondary_firewall_open "ensureSecondaryOpen" $OTHER_IPADDR
  	RC=$?
  	
  	if [ $RC -ne 0 ]; then	
			# We were not able to open the ports we need to bootstrap an
			# HA configuration on the secondary node.	
			return_error $RC "Error (2) opening JMS_HA firewall port on secondary node, RC=$RC"	
	fi
fi

if [ "$forceTimeSync" == "1" ]; then
	#ps auxww | grep -v grep | grep "/usr/sbin/ntpd"
	#ntpRunning=$?
	#if [ $ntpRunning == 0 ]; then
		# found ntp daemon, so need to end it
		#chnetcfg -s disable -c xntp 1> /dev/null 2>&1
	#fi
	
    times=`echo "HW: ";hwclock;echo "|| SYS: ";date`
    debug "Before NTP time sync: $times"    

	/usr/sbin/ntpd -q -g -u ntp:ntp -c /etc/ntp.conf

    # Now that the system clock is uptodate, force it back to the hwclock of the vm.
    times=`echo "HW: ";hwclock;echo "|| SYS: ";date`
    debug "After NTP time sync: $times"    
    /sbin/hwclock --systohc
    times=`echo "HW: ";hwclock;echo "|| SYS: ";date`
    debug "After --systohc: $times"    
    
	#if [ $ntpRunning == 0 ]; then
		# ntp daemon was running previously, so restart it
		#chnetcfg -s enable -c xntp 1> /dev/null 2>&1
	#fi

fi

if [ "$removeAllNTPServers" == "1" ]; then
	# get list of all servers in the /etc/ntp.conf file (except the local one, which starts with 127.)
	servers="`grep "^server " /etc/ntp.conf | sed -e 's/^server[ \t]*//' | grep -v "^127\." | cut -f 1 -d ' '`"

	# remove the servers, one by one
	for server in $servers
	do
		/usr/smrshbin/chnetcfg -s remove -c xntp -h $server 1>/dev/null 2>&1
	done
fi

if [ "$callSNTP" == "1" ]; then
	sntp $OTHER_HOSTNAME
fi

if [[ "$getVersion" == "1" ]] ; then
	#just call checkVersion below passing in the word "null" as the
	#remote version so it knows not to compare the versions
	checkVersion="null"
fi

if [[ ! -z $checkVersion ]] ; then
	localVersionLine=`LC_ALL=C /usr/smrshbin/lsconfig -V | grep "Version:"`
	localReleaseLine=`LC_ALL=C /usr/smrshbin/lsconfig -V | grep "Release:"`
	localServicePackLine=`LC_ALL=C /usr/smrshbin/lsconfig -V | grep "Service Pack:"`

	# get last part of the local*Lines since these are the values
	localVersion=`expr "$localVersionLine" : '.* \(.*\)'`
	localRelease=`expr "$localReleaseLine" : '.* \(.*\)'`
	localServicePack=`expr "$localServicePackLine" : '.* \(.*\)'`

	echo "$localVersion.$localRelease.$localServicePack"

	if [[ "$checkVersion" != "null" ]] ; then
		remoteVersion=`echo $checkVersion | cut -d . -f 1`
		remoteRelease=`echo $checkVersion | cut -d . -f 2`
		remoteServicePack=`echo $checkVersion | cut -d . -f 3`

		#Check if the two nodes have compatible versions.  This code 
		#can only decides if it supports previous versions.  It 
		#does not know if it is compatible with newer versions, the
		#newer version's code will decide that.

		if [[ remoteVersion -lt localVersion ]] ; then
			exit 1
		elif [[ remoteRelease -lt localRelease ]] ; then
			exit 2
		elif [[ remoteServicePack -lt localServicePack ]] ; then
			exit 3
		else
			exit 0
		fi
	fi
	
	exit 0
fi
	
