# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2009, 2011 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.

# Insertion sort
# AWK arrays start with 1
function isort(array, len) {
  for (i = 2; i < len; i++) {
    value = array[i]
    # The "0 +" forces numeric comparision
    for (j = i - 1; j >= 1 && 0 + array[j] > 0 + value; j--) {
      array[j + 1] = array[j]
      array[j] = value;
    }
  }
}

# Determine the number of directories there are
# ie.  / -> 0
# /tmp -> 1
# /usr/local/bin -> 3
function depth(str) {
  n = split(str, fields, "/")
  len = n

  for (i = 1; i <= n; i++) {
    if (fields[i] == "") {
      len-- 
    }
  }
  return len
}

# format of the output (all input comes in "<one>,<two>" form)
function print_output(out) {
  n = split(out, fields, ":")

  for (i = 1; i <= n; i++) {
    if (fields[i] != "") {
      split(fields[i], pair, ",")
      print pair[1] " " pair[2] " " pair[3]
    }
  }

}

BEGIN {
  FS=":"
  split(v, args)
  drive=args[1]
  prefix=args[2]
  upgr=args[3]
}

# All mount points that are partition numbers
!(/^#/ || /:swap/ || /:extended/ || /^$/ || /\/dump/) && /^[0-9]+/ {
  len = depth($2)

  if (upgr == "No") {
    # if we are not upgrading, then mark all partitions
    output[len] = output[len] ":m," drive $1 "," prefix $2
  } else {
    # if we are upgrading, then only print ones that are not marked as "no_up"
    if ($7 != "no_up") {
      output[len] = output[len] ":m," drive $1 "," prefix $2
    }
  }
}

# All bind mounts
!(/^#/ || /:swap/ || /:extended/ || /^$/ || /\/dump/) && !/^[0-9]+/ {
  len = depth($2)
  # if we are not upgrading then we will just display the sources
  if (upgr == "No") {
     output[len + 0.5] = output[len + 0.5] ":b," prefix $1 "," prefix $2
  }
}


END {
  # sort all the keys
  j = 1
  for (x in output) {
    keys[j++] = x
  }
  
  len = j;
  isort(keys, j)

  # With a sorted list of keys for the arrays, loop through in order
  for (j = 1; j < len; j++) {
    #print keys[i] " " output[keys[j]]
    print_output(output[keys[j]])
  }
}


