#!/bin/bash
# Licensed Materials - Property of IBM
#
# 09N6996
#   
# (C) Copyright IBM Corp. 2011 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.

SMHA_DEBUG_TSA=ON

source /opt/ibm/director/ha/bin/include/smha_exploiter_common.sh

#
# process OpState changes from the emulated HA resources
#

# validate the event

if [ -z "$ERRM_TYPEID" ]
then
    smha_fatal_error 9 "This script cannot be executed other than by ERRM"
fi

# the first argument to this script is the name of the "target" resource, i.e. the
# resource being locked offline to drive the TSA automation engine.

TARGET_RSRC_NAME=$1
if [ -z "$TARGET_RSRC_NAME" ]
then
    smha_fatal_error 9 "Missing target resource name argument"
fi

# format a time stamp from the event time

EpochSeconds=${ERRM_TIME%,*}
MicroSeconds=${ERRM_TIME#*,}

TS=$(date --date=@$EpochSeconds +%T).$MicroSeconds

if [ "$ERRM_TYPEID" = "2" ]           # if error event
then
    smha_debug_tsa "Received error event RH=$ERRM_RSRC_HANDLE $ERRM_ERROR_MSG"
    exit 0
fi

if [ "$ERRM_ATTR_PNAME" != "OpState" ]
then
    smha_fatal_error 9 "Received unexpected event at $TS for $ERRM_RSRC_CLASS_PNAME: $ERRM_EXPR"
fi

smha_debug_tsa "Received event at $TS for ${ERRM_RSRC_CLASS_PNAME}:${ERRM_RSRC_NAME} OpState=$ERRM_VALUE RH=$ERRM_RSRC_HANDLE"

export CT_MANAGEMENT_SCOPE=1

case $ERRM_VALUE in
    $TSA_UNKNOWN)
    ;;

    $TSA_ONLINE)
        if [ ${ERRM_RSRC_HANDLE:0:3} = "0x2" ]
        then
            # This is the event from the Aggregate resource. Reset the target
            # resource, which should be failed offline, if it is on this node.

            Result=$(lsrsrc-api -D + -s "IBM.Application::Name == '$TARGET_RSRC_NAME' && ResourceType == 0::ResourceHandle::OpState" 2> /dev/null)
            RH=${Result%+*}
            OpState=${Result#*+}

            if [ "$OpState" = "3" ]
            then
                # unlock the OpState value, but keep it offline

                /opt/ibm/director/ha/bin/test/smhaemctrl.sh ${TARGET_RSRC_NAME} offline

                # reset the resource so TSA will once again manage it
                /usr/sbin/rsct/bin/resetrsrc-api -r "$RH"
            fi
        else
            # This is the constituent resource. Check if it is on this node

            LocalNID=$(lsrsrc-api -s IBM.PeerNode::::NodeIDs 2> /dev/null)
            RsrcNID=$(CT_MANAGEMENT_SCOPE=2 lsrsrc-api -r "${ERRM_RSRC_HANDLE}::NodeIDs" 2> /dev/null)
            smha_debug_tsa "LocalNID=$LocalNID  RsrcNID=$RsrcNID"

            if [ "$RsrcNID" = "$LocalNID" ]
            then
                # Force target resource offline and keep it there, but wait a bit, first

                sleep 23
                /opt/ibm/director/ha/bin/test/smhaemctrl.sh ${TARGET_RSRC_NAME} Loffline
            fi
        fi
    ;;

    $TSA_OFFLINE)
    ;;

    $TSA_FAILED_OFFLINE)
    ;;

    $TSA_STUCK_ONLINE)
    ;;

    $TSA_PENDING_ONLINE)
    ;;

    $TSA_PENDING_OFFLINE)
    ;;
esac

exit 0
